@@ -18,13 +18,18 @@
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 // THE SOFTWARE.
 
-#include "Symbols.h"
-#include "Names.h"
-#include "TypeVisitor.h"
-#include "SymbolVisitor.h"
+#include "Control.h"
+#include "CoreTypes.h"
+#include "Literals.h"
 #include "Matcher.h"
+#include "Names.h"
 #include "Scope.h"
+#include "Symbols.h"
+#include "SymbolVisitor.h"
 #include "Templates.h"
+#include "TypeVisitor.h"
+
+#include <cstring>
 
 using namespace CPlusPlus;
 
@@ -99,7 +104,105 @@ Declaration::Declaration(Clone *clone, Subst *subst, Declaration *original)
     : Symbol(clone, subst, original)
     , _type(clone->type(original->_type, subst))
     , _initializer(clone->stringLiteral(original->_initializer))
-{ }
+{
+    const char* nameId = nullptr;
+    const Name *theName = name();
+    if (!theName)
+        return;
+
+    if (const Identifier* identifier = theName->identifier())
+        nameId = identifier->chars();
+    else
+        return;
+
+    Class *enClass = original->enclosingClass();
+    const char* enClassNameId = nullptr;
+    if (enClass && enClass->name() && enClass->name()->identifier()) {
+        enClassNameId = enClass->name()->identifier()->chars();
+    } else {
+        return;
+    }
+
+    if (!enClassNameId)
+        return;
+
+    Template *templSpec  = enClass->enclosingTemplate();
+    const char* enNamespaceNameId = nullptr;
+    if (templSpec) {
+        if (Namespace* ns = templSpec->enclosingNamespace()) {
+            if (ns->isInline())
+                ns = ns->enclosingNamespace();
+
+            if (ns->name() && ns->name()->identifier())
+                enNamespaceNameId =ns->name()->identifier()->chars();
+        }
+    }
+
+    if (!enNamespaceNameId || templSpec->templateParameterCount() < 1)
+        return;
+
+    const Name *firstTemplParamName = nullptr;
+    if (const TypenameArgument *templParam =
+            templSpec->templateParameterAt(0)->asTypenameArgument()) {
+        firstTemplParamName = templParam->name();
+    }
+
+    if (!firstTemplParamName)
+        return;
+
+    if (!subst)
+        return;
+
+    FullySpecifiedType newType;
+    if (std::strcmp(enNamespaceNameId, "std") == 0 ||
+        std::strcmp(enNamespaceNameId, "__cxx11") == 0) {
+        if (std::strcmp(enClassNameId, "unique_ptr") == 0) {
+            if (std::strcmp(nameId, "pointer") == 0) {
+                newType = clone->type(subst->apply(firstTemplParamName), 0);
+                newType = FullySpecifiedType(clone->control()->pointerType(newType));
+            }
+        } else if (std::strcmp(enClassNameId, "list") == 0 ||
+                   std::strcmp(enClassNameId, "forward_list") == 0 ||
+                   std::strcmp(enClassNameId, "vector") == 0 ||
+                   std::strcmp(enClassNameId, "queue") == 0 ||
+                   std::strcmp(enClassNameId, "deque") == 0 ||
+                   std::strcmp(enClassNameId, "set") == 0 ||
+                   std::strcmp(enClassNameId, "unordered_set") == 0 ||
+                   std::strcmp(enClassNameId, "multiset") == 0 ||
+                   std::strcmp(enClassNameId, "array") == 0) {
+            if (std::strcmp(nameId, "reference") == 0 ||
+                std::strcmp(nameId, "const_reference") == 0) {
+                newType = clone->type(subst->apply(firstTemplParamName), 0);
+            } else if (std::strcmp(nameId, "iterator") == 0 ||
+                       std::strcmp(nameId, "reverse_iterator") == 0 ||
+                       std::strcmp(nameId, "const_reverse_iterator") == 0 ||
+                       std::strcmp(nameId, "const_iterator") == 0) {
+                newType = clone->type(subst->apply(firstTemplParamName), 0);
+                newType = FullySpecifiedType(clone->control()->pointerType(newType));
+            }
+        } else if (std::strcmp(enClassNameId, "_Hash") == 0 ||
+                   std::strcmp(enClassNameId, "_Tree") == 0 ) {
+            if (std::strcmp(nameId, "iterator") == 0 ||
+                std::strcmp(nameId, "reverse_iterator") == 0 ||
+                std::strcmp(nameId, "const_reverse_iterator") == 0 ||
+                std::strcmp(nameId, "const_iterator") == 0) {
+                FullySpecifiedType clonedType = clone->type(subst->apply(firstTemplParamName), 0);
+                if (NamedType *namedType = clonedType.type()->asNamedType()) {
+                    if (const TemplateNameId * templateNameId =
+                            namedType->name()->asTemplateNameId()) {
+                        if (templateNameId->templateArgumentCount()) {
+                            newType = clone->type(templateNameId->templateArgumentAt(0), 0);
+                            newType = FullySpecifiedType(clone->control()->pointerType(newType));
+                        }
+                    }
+                }
+            }
+        }
+    }
+
+    if (newType.isValid())
+        _type = newType;
+}
 
 Declaration::~Declaration()
 { }
@@ -171,11 +274,13 @@ void Argument::visitSymbol0(SymbolVisitor *visitor)
 
 TypenameArgument::TypenameArgument(TranslationUnit *translationUnit, unsigned sourceLocation, const Name *name)
     : Symbol(translationUnit, sourceLocation, name)
+    , _isClassDeclarator(false)
 { }
 
 TypenameArgument::TypenameArgument(Clone *clone, Subst *subst, TypenameArgument *original)
     : Symbol(clone, subst, original)
     , _type(clone->type(original->_type, subst))
+    , _isClassDeclarator(original->_isClassDeclarator)
 { }
 
 TypenameArgument::~TypenameArgument()
@@ -222,7 +327,7 @@ int Function::methodKey() const
 void Function::setMethodKey(int key)
 { f._methodKey = key; }
 
-bool Function::isSignatureEqualTo(const Function *other) const
+bool Function::isSignatureEqualTo(const Function *other, Matcher *matcher) const
 {
     if (! other)
         return false;
@@ -230,30 +335,33 @@ bool Function::isSignatureEqualTo(const Function *other) const
         return false;
     else if (isVolatile() != other->isVolatile())
         return false;
-
-    const Name *l = unqualifiedName();
-    const Name *r = other->unqualifiedName();
-    if (l == r || (l && l->isEqualTo(r))) {
-        const unsigned argc = argumentCount();
-        if (argc != other->argumentCount())
-            return false;
-        for (unsigned i = 0; i < argc; ++i) {
-            Symbol *l = argumentAt(i);
-            Symbol *r = other->argumentAt(i);
-            if (! l->type().isEqualTo(r->type()))
-                return false;
-        }
-        return true;
-    }
-    return false;
-}
-
-bool Function::isEqualTo(const Type *other) const
-{
-    const Function *o = other->asFunctionType();
-    if (!isSignatureEqualTo(o))
+    else if (! Matcher::match(unqualifiedName(), other->unqualifiedName(), matcher))
         return false;
-    return _returnType.isEqualTo(o->_returnType);
+
+    const unsigned argc = argumentCount();
+    if (argc != other->argumentCount())
+        return false;
+    for (unsigned i = 0; i < argc; ++i) {
+        Symbol *l = argumentAt(i);
+        Symbol *r = other->argumentAt(i);
+        if (! l->type().match(r->type(), matcher)) {
+            if (!l->type()->isReferenceType() && !l->type()->isPointerType()
+                    && !l->type()->isPointerToMemberType()
+                    && !r->type()->isReferenceType() && !r->type()->isPointerType()
+                    && !r->type()->isPointerToMemberType()) {
+                FullySpecifiedType lType = l->type();
+                FullySpecifiedType rType = r->type();
+                lType.setConst(false);
+                lType.setVolatile(false);
+                rType.setConst(false);
+                rType.setVolatile(false);
+                if (lType.match(rType))
+                    continue;
+            }
+            return false;
+        }
+    }
+    return true;
 }
 
 void Function::accept0(TypeVisitor *visitor)
@@ -378,6 +486,12 @@ bool Function::isPureVirtual() const
 void Function::setPureVirtual(bool isPureVirtual)
 { f._isPureVirtual = isPureVirtual; }
 
+Function::RefQualifier Function::refQualifier() const
+{ return static_cast<RefQualifier>(f._refQualifier); }
+
+void Function::setRefQualifier(Function::RefQualifier refQualifier)
+{ f._refQualifier = refQualifier; }
+
 bool Function::isAmbiguous() const
 { return f._isAmbiguous; }
 
@@ -460,20 +574,6 @@ Enum::~Enum()
 FullySpecifiedType Enum::type() const
 { return FullySpecifiedType(const_cast<Enum *>(this)); }
 
-bool Enum::isEqualTo(const Type *other) const
-{
-    const Enum *o = other->asEnumType();
-    if (! o)
-        return false;
-    const Name *l = unqualifiedName();
-    const Name *r = o->unqualifiedName();
-    if (l == r)
-        return true;
-    else if (! l)
-        return false;
-    return l->isEqualTo(r);
-}
-
 bool Enum::isScoped() const
 {
     return _isScoped;
@@ -543,9 +643,6 @@ Symbol *Template::declaration() const
 FullySpecifiedType Template::type() const
 { return FullySpecifiedType(const_cast<Template *>(this)); }
 
-bool Template::isEqualTo(const Type *other) const
-{ return other == this; }
-
 void Template::visitSymbol0(SymbolVisitor *visitor)
 {
     if (visitor->visit(this)) {
@@ -578,18 +675,6 @@ Namespace::Namespace(Clone *clone, Subst *subst, Namespace *original)
 Namespace::~Namespace()
 { }
 
-bool Namespace::isEqualTo(const Type *other) const
-{
-    const Namespace *o = other->asNamespaceType();
-    if (! o)
-        return false;
-    const Name *l = unqualifiedName();
-    const Name *r = o->unqualifiedName();
-    if (l == r || (l && l->isEqualTo(r)))
-        return true;
-    return false;
-}
-
 void Namespace::accept0(TypeVisitor *visitor)
 { visitor->visit(this); }
 
@@ -639,6 +724,12 @@ bool BaseClass::isVirtual() const
 void BaseClass::setVirtual(bool isVirtual)
 { _isVirtual = isVirtual; }
 
+bool BaseClass::isVariadic() const
+{ return _isVariadic; }
+
+void BaseClass::setVariadic(bool isVariadic)
+{ _isVariadic = isVariadic; }
+
 void BaseClass::visitSymbol0(SymbolVisitor *visitor)
 { visitor->visit(this); }
 
@@ -657,19 +748,6 @@ ForwardClassDeclaration::~ForwardClassDeclaration()
 FullySpecifiedType ForwardClassDeclaration::type() const
 { return FullySpecifiedType(const_cast<ForwardClassDeclaration *>(this)); }
 
-bool ForwardClassDeclaration::isEqualTo(const Type *other) const
-{
-    if (const ForwardClassDeclaration *otherClassFwdTy = other->asForwardClassDeclarationType()) {
-        if (name() == otherClassFwdTy->name())
-            return true;
-        else if (name() && otherClassFwdTy->name())
-            return name()->isEqualTo(otherClassFwdTy->name());
-
-        return false;
-    }
-    return false;
-}
-
 void ForwardClassDeclaration::visitSymbol0(SymbolVisitor *visitor)
 { visitor->visit(this); }
 
@@ -738,19 +816,6 @@ void Class::addBaseClass(BaseClass *baseClass)
 FullySpecifiedType Class::type() const
 { return FullySpecifiedType(const_cast<Class *>(this)); }
 
-bool Class::isEqualTo(const Type *other) const
-{
-    const Class *o = other->asClassType();
-    if (! o)
-        return false;
-    const Name *l = unqualifiedName();
-    const Name *r = o->unqualifiedName();
-    if (l == r || (l && l->isEqualTo(r)))
-        return true;
-    else
-        return false;
-}
-
 void Class::visitSymbol0(SymbolVisitor *visitor)
 {
     if (visitor->visit(this)) {
@@ -901,20 +966,6 @@ void ObjCClass::addProtocol(ObjCBaseProtocol *protocol)
 FullySpecifiedType ObjCClass::type() const
 { return FullySpecifiedType(const_cast<ObjCClass *>(this)); }
 
-bool ObjCClass::isEqualTo(const Type *other) const
-{
-    const ObjCClass *o = other->asObjCClassType();
-    if (!o)
-        return false;
-
-    const Name *l = unqualifiedName();
-    const Name *r = o->unqualifiedName();
-    if (l == r || (l && l->isEqualTo(r)))
-        return true;
-    else
-        return false;
-}
-
 void ObjCClass::visitSymbol0(SymbolVisitor *visitor)
 {
     if (visitor->visit(this)) {
@@ -967,20 +1018,6 @@ void ObjCProtocol::addProtocol(ObjCBaseProtocol *protocol)
 FullySpecifiedType ObjCProtocol::type() const
 { return FullySpecifiedType(const_cast<ObjCProtocol *>(this)); }
 
-bool ObjCProtocol::isEqualTo(const Type *other) const
-{
-    const ObjCProtocol *o = other->asObjCProtocolType();
-    if (!o)
-        return false;
-
-    const Name *l = unqualifiedName();
-    const Name *r = o->unqualifiedName();
-    if (l == r || (l && l->isEqualTo(r)))
-        return true;
-    else
-        return false;
-}
-
 void ObjCProtocol::visitSymbol0(SymbolVisitor *visitor)
 {
     if (visitor->visit(this)) {
@@ -1016,20 +1053,6 @@ ObjCForwardClassDeclaration::~ObjCForwardClassDeclaration()
 FullySpecifiedType ObjCForwardClassDeclaration::type() const
 { return FullySpecifiedType(); }
 
-bool ObjCForwardClassDeclaration::isEqualTo(const Type *other) const
-{
-    if (const ObjCForwardClassDeclaration *otherFwdClass = other->asObjCForwardClassDeclarationType()) {
-        if (name() == otherFwdClass->name())
-            return true;
-        else if (name() && otherFwdClass->name())
-            return name()->isEqualTo(otherFwdClass->name());
-        else
-            return false;
-    }
-
-    return false;
-}
-
 void ObjCForwardClassDeclaration::visitSymbol0(SymbolVisitor *visitor)
 { visitor->visit(this); }
 
@@ -1060,20 +1083,6 @@ ObjCForwardProtocolDeclaration::~ObjCForwardProtocolDeclaration()
 FullySpecifiedType ObjCForwardProtocolDeclaration::type() const
 { return FullySpecifiedType(); }
 
-bool ObjCForwardProtocolDeclaration::isEqualTo(const Type *other) const
-{
-    if (const ObjCForwardProtocolDeclaration *otherFwdProtocol = other->asObjCForwardProtocolDeclarationType()) {
-        if (name() == otherFwdProtocol->name())
-            return true;
-        else if (name() && otherFwdProtocol->name())
-            return name()->isEqualTo(otherFwdProtocol->name());
-        else
-            return false;
-    }
-
-    return false;
-}
-
 void ObjCForwardProtocolDeclaration::visitSymbol0(SymbolVisitor *visitor)
 { visitor->visit(this); }
 
@@ -1102,30 +1111,6 @@ ObjCMethod::ObjCMethod(Clone *clone, Subst *subst, ObjCMethod *original)
 ObjCMethod::~ObjCMethod()
 { }
 
-bool ObjCMethod::isEqualTo(const Type *other) const
-{
-    const ObjCMethod *o = other->asObjCMethodType();
-    if (! o)
-        return false;
-
-    const Name *l = unqualifiedName();
-    const Name *r = o->unqualifiedName();
-    if (l == r || (l && l->isEqualTo(r))) {
-        if (argumentCount() != o->argumentCount())
-            return false;
-        else if (! _returnType.isEqualTo(o->_returnType))
-            return false;
-        for (unsigned i = 0; i < argumentCount(); ++i) {
-            Symbol *l = argumentAt(i);
-            Symbol *r = o->argumentAt(i);
-            if (! l->type().isEqualTo(r->type()))
-                return false;
-        }
-        return true;
-    }
-    return false;
-}
-
 void ObjCMethod::accept0(TypeVisitor *visitor)
 { visitor->visit(this); }
 
