@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,34 +9,34 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company.  For licensing terms and
-** conditions see http://www.qt.io/terms-conditions.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, The Qt Company gives you certain additional
-** rights.  These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
 #include "androiddevicedialog.h"
 #include "androidmanager.h"
+#include "androidavdmanager.h"
+#include "avddialog.h"
 #include "ui_androiddevicedialog.h"
 
+#include <utils/environment.h>
+#include <utils/progressindicator.h>
+#include <utils/algorithm.h>
+
 #include <QMessageBox>
 #include <QPainter>
 #include <QStyledItemDelegate>
 #include <QToolTip>
-#include <QtConcurrentRun>
 
 using namespace Android;
 using namespace Android::Internal;
@@ -105,24 +105,23 @@ private:
 
 class AndroidDeviceModelDelegate : public QStyledItemDelegate
 {
+    Q_OBJECT
 public:
-    AndroidDeviceModelDelegate(QObject * parent = 0)
+    AndroidDeviceModelDelegate(QObject * parent = nullptr)
         : QStyledItemDelegate(parent)
     {
 
     }
 
-    ~AndroidDeviceModelDelegate()
-    {
-    }
+    ~AndroidDeviceModelDelegate() override = default;
 
-    void paint(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const
+    void paint(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const override
     {
-        QStyleOptionViewItemV4 opt = option;
+        QStyleOptionViewItem opt = option;
         initStyleOption(&opt, index);
         painter->save();
 
-        AndroidDeviceModelNode *node = static_cast<AndroidDeviceModelNode *>(index.internalPointer());
+        auto node = static_cast<AndroidDeviceModelNode *>(index.internalPointer());
         AndroidDeviceInfo device = node->deviceInfo();
 
         painter->setPen(Qt::NoPen);
@@ -132,7 +131,7 @@ public:
         palette.setCurrentColorGroup(QPalette::Active);
         bool selected = opt.state & QStyle::State_Selected;
         QColor backgroundColor = selected ? palette.highlight().color()
-                                          : palette.background().color();
+                                          : palette.window().color();
         painter->setBrush(backgroundColor);
 
         painter->drawRect(0, opt.rect.top(), opt.rect.width() + opt.rect.left(), opt.rect.height());
@@ -164,15 +163,35 @@ public:
 
             QFontMetrics fm(opt.font);
             // TopLeft
-            QString topLeft = device.serialNumber;
+            QString topLeft;
             if (device.type == AndroidDeviceInfo::Hardware)
                 topLeft = AndroidConfigurations::currentConfig().getProductModel(device.serialNumber);
+            else
+                topLeft = device.avdname;
             painter->drawText(size + 12, 2 + opt.rect.top() + fm.ascent(), topLeft);
 
-            QString topRight = device.serialNumber;
+
             // topRight
-            if (device.type == AndroidDeviceInfo::Hardware) // otherwise it's not very informative
-                painter->drawText(opt.rect.right() - fm.width(topRight) - 6 , 2 + opt.rect.top() + fm.ascent(), topRight);
+            auto drawTopRight = [&](const QString text, const QFontMetrics &fm) {
+                painter->drawText(opt.rect.right() - fm.width(text) - 6 , 2 + opt.rect.top() + fm.ascent(), text);
+            };
+
+            if (device.type == AndroidDeviceInfo::Hardware) {
+                drawTopRight(device.serialNumber, fm);
+            } else {
+                AndroidConfig::OpenGl openGl = AndroidConfigurations::currentConfig().getOpenGLEnabled(device.avdname);
+                if (openGl == AndroidConfig::OpenGl::Enabled) {
+                    drawTopRight(tr("OpenGL enabled"), fm);
+                } else if (openGl == AndroidConfig::OpenGl::Disabled) {
+                    QFont font = painter->font();
+                    font.setBold(true);
+                    painter->setFont(font);
+                    QFontMetrics fmBold(font);
+                    drawTopRight(tr("OpenGL disabled"), fmBold);
+                    font.setBold(false);
+                    painter->setFont(font);
+                }
+            }
 
             // Directory
             QColor mix;
@@ -187,6 +206,9 @@ public:
                 lineText += AndroidDeviceDialog::tr("ABI:") + device.cpuAbi.join(QLatin1Char(' '));
             } else {
                 lineText = node->incompatibleReason();
+                QFont f = painter->font();
+                f.setBold(true);
+                painter->setFont(f);
             }
             painter->drawText(size + 12, opt.rect.top() + fm.ascent() + fm.height() + 6, lineText);
         }
@@ -197,9 +219,9 @@ public:
         painter->restore();
     }
 
-    QSize sizeHint(const QStyleOptionViewItem &option, const QModelIndex &index) const
+    QSize sizeHint(const QStyleOptionViewItem &option, const QModelIndex &index) const override
     {
-        QStyleOptionViewItemV4 opt = option;
+        QStyleOptionViewItem opt = option;
         initStyleOption(&opt, index);
 
         QFontMetrics fm(option.font);
@@ -212,24 +234,24 @@ public:
 
 class AndroidDeviceModel : public QAbstractItemModel
 {
+    Q_OBJECT
 public:
-    AndroidDeviceModel(int apiLevel, const QString &abi, AndroidConfigurations::Options options);
+    AndroidDeviceModel(int apiLevel, const QString &abi);
     QModelIndex index(int row, int column,
-                      const QModelIndex &parent = QModelIndex()) const;
-    QModelIndex parent(const QModelIndex &child) const;
-    int rowCount(const QModelIndex &parent = QModelIndex()) const;
-    int columnCount(const QModelIndex &parent = QModelIndex()) const;
-    QVariant data(const QModelIndex &index, int role = Qt::DisplayRole) const;
-    Qt::ItemFlags flags(const QModelIndex &index) const;
+                      const QModelIndex &parent = QModelIndex()) const override;
+    QModelIndex parent(const QModelIndex &child) const override;
+    int rowCount(const QModelIndex &parent = QModelIndex()) const override;
+    int columnCount(const QModelIndex &parent = QModelIndex()) const override;
+    QVariant data(const QModelIndex &index, int role = Qt::DisplayRole) const override;
+    Qt::ItemFlags flags(const QModelIndex &index) const override;
 
     AndroidDeviceInfo device(QModelIndex index);
     void setDevices(const QVector<AndroidDeviceInfo> &devices);
 
-    QModelIndex indexFor(const QString &serial);
+    QModelIndex indexFor(AndroidDeviceInfo::AndroidDeviceType type, const QString &serial);
 private:
     int m_apiLevel;
     QString m_abi;
-    AndroidConfigurations::Options m_options;
     AndroidDeviceModelNode *m_root;
 };
 
@@ -238,8 +260,8 @@ private:
 /////////////////
 // AndroidDeviceModel
 /////////////////
-AndroidDeviceModel::AndroidDeviceModel(int apiLevel, const QString &abi, AndroidConfigurations::Options options)
-    : m_apiLevel(apiLevel), m_abi(abi), m_options(options), m_root(0)
+AndroidDeviceModel::AndroidDeviceModel(int apiLevel, const QString &abi)
+    : m_apiLevel(apiLevel), m_abi(abi), m_root(nullptr)
 {
 }
 
@@ -257,7 +279,7 @@ QModelIndex AndroidDeviceModel::index(int row, int column, const QModelIndex &pa
         return createIndex(row, column, m_root->children().at(row));
     }
 
-    AndroidDeviceModelNode *node = static_cast<AndroidDeviceModelNode *>(parent.internalPointer());
+    auto node = static_cast<AndroidDeviceModelNode *>(parent.internalPointer());
     if (row < node->children().count())
         return createIndex(row, column, node->children().at(row));
 
@@ -270,7 +292,7 @@ QModelIndex AndroidDeviceModel::parent(const QModelIndex &child) const
         return QModelIndex();
     if (!m_root)
         return QModelIndex();
-    AndroidDeviceModelNode *node = static_cast<AndroidDeviceModelNode *>(child.internalPointer());
+    auto node = static_cast<AndroidDeviceModelNode *>(child.internalPointer());
     if (node == m_root)
         return QModelIndex();
     AndroidDeviceModelNode *parent = node->parent();
@@ -288,7 +310,7 @@ int AndroidDeviceModel::rowCount(const QModelIndex &parent) const
         return 0;
     if (!parent.isValid())
         return m_root->children().count();
-    AndroidDeviceModelNode *node = static_cast<AndroidDeviceModelNode *>(parent.internalPointer());
+    auto node = static_cast<AndroidDeviceModelNode *>(parent.internalPointer());
     return node->children().count();
 }
 
@@ -302,7 +324,7 @@ QVariant AndroidDeviceModel::data(const QModelIndex &index, int role) const
 {
     if (role != Qt::DisplayRole)
         return QVariant();
-    AndroidDeviceModelNode *node = static_cast<AndroidDeviceModelNode *>(index.internalPointer());
+    auto node = static_cast<AndroidDeviceModelNode *>(index.internalPointer());
     if (!node)
         return QVariant();
     return node->deviceInfo().serialNumber;
@@ -310,7 +332,7 @@ QVariant AndroidDeviceModel::data(const QModelIndex &index, int role) const
 
 Qt::ItemFlags AndroidDeviceModel::flags(const QModelIndex &index) const
 {
-    AndroidDeviceModelNode *node = static_cast<AndroidDeviceModelNode *>(index.internalPointer());
+    auto node = static_cast<AndroidDeviceModelNode *>(index.internalPointer());
     if (node)
         if (node->displayName().isEmpty() && node->incompatibleReason().isEmpty())
             return Qt::ItemIsSelectable | Qt::ItemIsEnabled;
@@ -320,7 +342,7 @@ Qt::ItemFlags AndroidDeviceModel::flags(const QModelIndex &index) const
 
 AndroidDeviceInfo AndroidDeviceModel::device(QModelIndex index)
 {
-    AndroidDeviceModelNode *node = static_cast<AndroidDeviceModelNode *>(index.internalPointer());
+    auto node = static_cast<AndroidDeviceModelNode *>(index.internalPointer());
     if (!node)
         return AndroidDeviceInfo();
     return node->deviceInfo();
@@ -330,10 +352,10 @@ void AndroidDeviceModel::setDevices(const QVector<AndroidDeviceInfo> &devices)
 {
     beginResetModel();
     delete m_root;
-    m_root = new AndroidDeviceModelNode(0, QString());
+    m_root = new AndroidDeviceModelNode(nullptr, QString());
 
     AndroidDeviceModelNode *compatibleDevices = new AndroidDeviceModelNode(m_root, AndroidDeviceDialog::tr("Compatible devices"));
-    AndroidDeviceModelNode *incompatibleDevices = 0; // created on demand
+    AndroidDeviceModelNode *incompatibleDevices = nullptr; // created on demand
     foreach (const AndroidDeviceInfo &device, devices) {
         QString error;
         if (device.state == AndroidDeviceInfo::UnAuthorizedState) {
@@ -348,8 +370,6 @@ void AndroidDeviceModel::setDevices(const QVector<AndroidDeviceInfo> &devices)
         } else if (device.sdk < m_apiLevel) {
             error = AndroidDeviceDialog::tr("API Level of device is: %1.")
                     .arg(device.sdk);
-        } else if (device.sdk > 20 && (m_options & AndroidConfigurations::FilterAndroid5)) {
-            error = AndroidDeviceDialog::tr("Android 5 devices are incompatible with deploying Qt to a temporary directory.");
         } else {
             new AndroidDeviceModelNode(compatibleDevices, device);
             continue;
@@ -361,12 +381,16 @@ void AndroidDeviceModel::setDevices(const QVector<AndroidDeviceInfo> &devices)
     endResetModel();
 }
 
-QModelIndex AndroidDeviceModel::indexFor(const QString &serial)
+QModelIndex AndroidDeviceModel::indexFor(AndroidDeviceInfo::AndroidDeviceType type, const QString &serial)
 {
     foreach (AndroidDeviceModelNode *topLevelNode, m_root->children()) {
         QList<AndroidDeviceModelNode *> deviceNodes = topLevelNode->children();
         for (int i = 0; i < deviceNodes.size(); ++i) {
-            if (deviceNodes.at(i)->deviceInfo().serialNumber == serial)
+            const AndroidDeviceInfo &info = deviceNodes.at(i)->deviceInfo();
+            if (info.type != type)
+                continue;
+            if ((type == AndroidDeviceInfo::Hardware && serial == info.serialNumber)
+                    || (type == AndroidDeviceInfo::Emulator && serial == info.avdname))
                 return createIndex(i, 0, deviceNodes.at(i));
         }
     }
@@ -389,12 +413,15 @@ static inline QString msgAdbListDevices()
     return AndroidDeviceDialog::tr("<p>The adb tool in the Android SDK lists all connected devices if run via &quot;adb devices&quot;.</p>");
 }
 
-AndroidDeviceDialog::AndroidDeviceDialog(int apiLevel, const QString &abi, AndroidConfigurations::Options options, QWidget *parent) :
+AndroidDeviceDialog::AndroidDeviceDialog(int apiLevel, const QString &abi,
+                                         const QString &serialNumber, QWidget *parent) :
     QDialog(parent),
-    m_model(new AndroidDeviceModel(apiLevel, abi, options)),
+    m_model(new AndroidDeviceModel(apiLevel, abi)),
     m_ui(new Ui::AndroidDeviceDialog),
     m_apiLevel(apiLevel),
-    m_abi(abi)
+    m_abi(abi),
+    m_defaultDevice(serialNumber),
+    m_avdManager(new AndroidAvdManager)
 {
     m_ui->setupUi(this);
     m_ui->deviceView->setModel(m_model);
@@ -404,66 +431,112 @@ AndroidDeviceDialog::AndroidDeviceDialog(int apiLevel, const QString &abi, Andro
     m_ui->deviceView->setUniformRowHeights(true);
     m_ui->deviceView->setExpandsOnDoubleClick(false);
 
-    m_ui->defaultDeviceCheckBox->setText(tr("Always use this device for architecture %1").arg(abi));
+    m_ui->defaultDeviceCheckBox->setText(tr("Always use this device for architecture %1 for this project").arg(abi));
 
     m_ui->noDeviceFoundLabel->setText(QLatin1String("<p align=\"center\"><span style=\" font-size:16pt;\">")
                                       + tr("No Device Found") + QLatin1String("</span></p><br/>")
                                       + msgConnect() + QLatin1String("<br/>")
                                       + msgAdbListDevices());
-    connect(m_ui->missingLabel, SIGNAL(linkActivated(QString)),
-            this, SLOT(showHelp()));
+    connect(m_ui->missingLabel, &QLabel::linkActivated,
+            this, &AndroidDeviceDialog::showHelp);
 
-    connect(m_ui->refreshDevicesButton, SIGNAL(clicked()),
-            this, SLOT(refreshDeviceList()));
+    connect(m_ui->refreshDevicesButton, &QAbstractButton::clicked,
+            this, &AndroidDeviceDialog::refreshDeviceList);
 
-    connect(m_ui->createAVDButton, SIGNAL(clicked()),
-            this, SLOT(createAvd()));
-    connect(m_ui->deviceView, SIGNAL(doubleClicked(QModelIndex)),
-            this, SLOT(accept()));
+    connect(m_ui->createAVDButton, &QAbstractButton::clicked,
+            this, &AndroidDeviceDialog::createAvd);
+    connect(m_ui->deviceView, &QAbstractItemView::doubleClicked,
+            this, &QDialog::accept);
 
-    connect(&m_futureWatcher, SIGNAL(finished()),
-            this, SLOT(avdAdded()));
+    connect(&m_futureWatcherAddDevice, &QFutureWatcherBase::finished,
+            this, &AndroidDeviceDialog::avdAdded);
+    connect(&m_futureWatcherRefreshDevices, &QFutureWatcherBase::finished,
+            this, &AndroidDeviceDialog::devicesRefreshed);
 
-    refreshDeviceList();
+    connect(m_ui->deviceView->selectionModel(), &QItemSelectionModel::currentChanged,
+            this, &AndroidDeviceDialog::enableOkayButton);
+
+    m_ui->buttonBox->button(QDialogButtonBox::Ok)->setEnabled(false);
+
+    m_progressIndicator = new Utils::ProgressIndicator(Utils::ProgressIndicatorSize::Large, this);
+    m_progressIndicator->attachToWidget(m_ui->deviceView);
+
+    if (serialNumber.isEmpty()) {
+        m_ui->lookingForDevice->setVisible(false);
+        m_ui->lookingForDeviceCancel->setVisible(false);
+    } else {
+        m_ui->lookingForDevice->setVisible(true);
+        m_ui->lookingForDevice->setText(tr("Looking for default device <b>%1</b>.").arg(serialNumber));
+        m_ui->lookingForDeviceCancel->setVisible(true);
+    }
+
+    connect(m_ui->lookingForDeviceCancel, &QPushButton::clicked,
+            this, &AndroidDeviceDialog::defaultDeviceClear);
+
+    m_connectedDevices = AndroidConfig::connectedDevices(AndroidConfigurations::currentConfig().adbToolPath().toString());
 }
 
 AndroidDeviceDialog::~AndroidDeviceDialog()
 {
+    m_futureWatcherAddDevice.waitForFinished();
+    m_futureWatcherRefreshDevices.waitForFinished();
     delete m_ui;
-    m_futureWatcher.waitForFinished();
 }
 
 AndroidDeviceInfo AndroidDeviceDialog::device()
 {
-    if (result() == QDialog::Accepted)
+    if (!m_defaultDevice.isEmpty()) {
+        auto device = std::find_if(m_connectedDevices.begin(), m_connectedDevices.end(), [this](const AndroidDeviceInfo& info) {
+            return info.serialNumber == m_defaultDevice ||
+                    info.avdname == m_defaultDevice;
+        });
+
+        if (device != m_connectedDevices.end())
+            return *device;
+        m_defaultDevice.clear();
+    }
+
+    refreshDeviceList();
+
+    if (exec() == QDialog::Accepted)
         return m_model->device(m_ui->deviceView->currentIndex());
     return AndroidDeviceInfo();
 }
 
-void AndroidDeviceDialog::accept()
-{
-    QDialog::accept();
-}
-
-bool AndroidDeviceDialog::saveDeviceSelection()
+bool AndroidDeviceDialog::saveDeviceSelection() const
 {
     return m_ui->defaultDeviceCheckBox->isChecked();
 }
 
 void AndroidDeviceDialog::refreshDeviceList()
 {
+    m_ui->refreshDevicesButton->setEnabled(false);
+    m_progressIndicator->show();
+    m_connectedDevices = AndroidConfig::connectedDevices(AndroidConfigurations::currentConfig().adbToolPath().toString());
+    m_futureWatcherRefreshDevices.setFuture(m_avdManager->avdList());
+}
+
+void AndroidDeviceDialog::devicesRefreshed()
+{
+    m_progressIndicator->hide();
     QString serialNumber;
+    AndroidDeviceInfo::AndroidDeviceType deviceType = AndroidDeviceInfo::Hardware;
     QModelIndex currentIndex = m_ui->deviceView->currentIndex();
-    if (currentIndex.isValid())
-        serialNumber = m_model->device(currentIndex).serialNumber;
+    if (currentIndex.isValid()) { // save currently selected index
+        AndroidDeviceInfo info = m_model->device(currentIndex);
+        deviceType = info.type;
+        serialNumber = deviceType == AndroidDeviceInfo::Hardware ? info.serialNumber : info.avdname;
+    }
 
-    QVector<AndroidDeviceInfo> devices;
-    foreach (const AndroidDeviceInfo &info, AndroidConfigurations::currentConfig().connectedDevices())
-        if (info.type == AndroidDeviceInfo::Hardware)
-            devices << info;
+    AndroidDeviceInfoList devices = m_futureWatcherRefreshDevices.result();
+    QSet<QString> startedAvds = Utils::transform<QSet>(m_connectedDevices,
+                                                       &AndroidDeviceInfo::avdname);
 
-    devices += AndroidConfigurations::currentConfig().androidVirtualDevices();
-    m_model->setDevices(devices);
+    for (const AndroidDeviceInfo &dev : devices)
+        if (!startedAvds.contains(dev.avdname))
+            m_connectedDevices << dev;
+
+    m_model->setDevices(m_connectedDevices);
 
     m_ui->deviceView->expand(m_model->index(0, 0));
     if (m_model->rowCount() > 1) // we have a incompatible device node
@@ -471,49 +544,75 @@ void AndroidDeviceDialog::refreshDeviceList()
 
     // Smartly select a index
     QModelIndex newIndex;
-    if (!serialNumber.isEmpty())
-        newIndex = m_model->indexFor(serialNumber);
+    if (!m_defaultDevice.isEmpty()) {
+        newIndex = m_model->indexFor(AndroidDeviceInfo::Hardware, m_defaultDevice);
+        if (!newIndex.isValid())
+            newIndex = m_model->indexFor(AndroidDeviceInfo::Emulator, m_defaultDevice);
+        if (!newIndex.isValid()) // not found the default device
+            defaultDeviceClear();
+    }
 
-    if (!newIndex.isValid() && !devices.isEmpty())
-        newIndex = m_model->indexFor(devices.first().serialNumber);
+    if (!newIndex.isValid() && !m_avdNameFromAdd.isEmpty()) {
+        newIndex = m_model->indexFor(AndroidDeviceInfo::Emulator, m_avdNameFromAdd);
+        m_avdNameFromAdd.clear();
+    }
+
+    if (!newIndex.isValid() && !serialNumber.isEmpty())
+        newIndex = m_model->indexFor(deviceType, serialNumber);
+
+    if (!newIndex.isValid() && !m_connectedDevices.isEmpty()) {
+        AndroidDeviceInfo info = m_connectedDevices.first();
+        const QString &name = info.type == AndroidDeviceInfo::Hardware ? info.serialNumber : info.avdname;
+        newIndex = m_model->indexFor(info.type, name);
+    }
 
     m_ui->deviceView->setCurrentIndex(newIndex);
 
-    m_ui->stackedWidget->setCurrentIndex(devices.isEmpty() ? 1 : 0);
+    m_ui->stackedWidget->setCurrentIndex(m_connectedDevices.isEmpty() ? 1 : 0);
+
+    m_ui->refreshDevicesButton->setEnabled(true);
+    m_connectedDevices.clear();
 }
 
 void AndroidDeviceDialog::createAvd()
 {
     m_ui->createAVDButton->setEnabled(false);
-    AndroidConfig::CreateAvdInfo info = AndroidConfigurations::currentConfig().gatherCreateAVDInfo(this, m_apiLevel, m_abi);
+    CreateAvdInfo info = AvdDialog::gatherCreateAVDInfo(this, AndroidConfigurations::sdkManager(),
+                                                        m_apiLevel, m_abi);
 
-    if (info.target.isEmpty()) {
+    if (!info.isValid()) {
         m_ui->createAVDButton->setEnabled(true);
         return;
     }
 
-    m_futureWatcher.setFuture(AndroidConfigurations::currentConfig().createAVD(info));
+    m_futureWatcherAddDevice.setFuture(m_avdManager->createAvd(info));
 }
 
 void AndroidDeviceDialog::avdAdded()
 {
     m_ui->createAVDButton->setEnabled(true);
-    AndroidConfig::CreateAvdInfo info = m_futureWatcher.result();
+    CreateAvdInfo info = m_futureWatcherAddDevice.result();
     if (!info.error.isEmpty()) {
         QMessageBox::critical(this, QApplication::translate("AndroidConfig", "Error Creating AVD"), info.error);
         return;
     }
 
+    m_avdNameFromAdd = info.name;
     refreshDeviceList();
-    QModelIndex index = m_model->indexFor(info.name);
-    m_ui->deviceView->setCurrentIndex(index);
+}
+
+void AndroidDeviceDialog::enableOkayButton()
+{
+    AndroidDeviceModelNode *node = static_cast<AndroidDeviceModelNode *>(m_ui->deviceView->currentIndex().internalPointer());
+    bool enable = node && (!node->deviceInfo().serialNumber.isEmpty() || !node->deviceInfo().avdname.isEmpty());
+    m_ui->buttonBox->button(QDialogButtonBox::Ok)->setEnabled(enable);
 }
 
 // Does not work.
 void AndroidDeviceDialog::clickedOnView(const QModelIndex &idx)
 {
     if (idx.isValid()) {
-        AndroidDeviceModelNode *node = static_cast<AndroidDeviceModelNode *>(idx.internalPointer());
+        auto node = static_cast<AndroidDeviceModelNode *>(idx.internalPointer());
         if (!node->displayName().isEmpty()) {
             if (m_ui->deviceView->isExpanded(idx))
                 m_ui->deviceView->collapse(idx);
@@ -529,3 +628,12 @@ void AndroidDeviceDialog::showHelp()
     pos = m_ui->missingLabel->parentWidget()->mapToGlobal(pos);
     QToolTip::showText(pos, msgConnect() + msgAdbListDevices(), this);
 }
+
+void AndroidDeviceDialog::defaultDeviceClear()
+{
+    m_ui->lookingForDevice->setVisible(false);
+    m_ui->lookingForDeviceCancel->setVisible(false);
+    m_defaultDevice.clear();
+}
+
+#include "androiddevicedialog.moc"
