@@ -33,19 +33,19 @@ namespace Core {
 
 class ILocatorFilter;
 
-namespace Internal { class LocatorWidget; }
-
 class CORE_EXPORT LocatorManager : public QObject
 {
     Q_OBJECT
 
 public:
-    LocatorManager(QObject *parent = nullptr);
+    LocatorManager();
 
     static void showFilter(ILocatorFilter *filter);
     static void show(const QString &text, int selectionStart = -1, int selectionLength = 0);
 
     static QWidget *createLocatorInputWidget(QWidget *window);
+
+    static bool locatorHasFocus();
 };
 
 } // namespace Core
