@@ -30,6 +30,7 @@
 #include "icore.h"
 #include "ioutputpane.h"
 #include "modemanager.h"
+#include "statusbarmanager.h"
 
 #include <coreplugin/actionmanager/actionmanager.h>
 #include <coreplugin/actionmanager/actioncontainer.h>
@@ -62,30 +63,51 @@
 #include <QTimeLine>
 
 using namespace Utils;
+using namespace Core::Internal;
 
 namespace Core {
+namespace Internal {
+
+class OutputPaneData
+{
+public:
+    OutputPaneData(IOutputPane *pane = nullptr) : pane(pane) {}
+
+    IOutputPane *pane = nullptr;
+    Id id;
+    OutputPaneToggleButton *button = nullptr;
+    QAction *action = nullptr;
+    bool buttonVisible = false;
+};
+
+static QVector<OutputPaneData> g_outputPanes;
+static bool g_managerConstructed = false; // For debugging reasons.
+
+} // Internal
 
 // OutputPane
 
-static QList<IOutputPane *> g_outputPanes;
-
 IOutputPane::IOutputPane(QObject *parent)
     : QObject(parent)
 {
-    g_outputPanes.append(this);
+    // We need all pages first. Ignore latecomers and shout.
+    QTC_ASSERT(!g_managerConstructed, return);
+    g_outputPanes.append(OutputPaneData(this));
 }
 
 IOutputPane::~IOutputPane()
 {
-    g_outputPanes.removeOne(this);
+    const int i = Utils::indexOf(g_outputPanes, Utils::equal(&OutputPaneData::pane, this));
+    QTC_ASSERT(i >= 0, return);
+    delete g_outputPanes.at(i).button;
 }
 
 namespace Internal {
 
-static char outputPaneSettingsKeyC[] = "OutputPaneVisibility";
-static char outputPaneIdKeyC[] = "id";
-static char outputPaneVisibleKeyC[] = "visible";
-static const int buttonBorderWidth = 3;
+const char outputPaneSettingsKeyC[] = "OutputPaneVisibility";
+const char outputPaneIdKeyC[] = "id";
+const char outputPaneVisibleKeyC[] = "visible";
+const int buttonBorderWidth = 3;
 
 static int numberAreaWidth()
 {
@@ -96,7 +118,7 @@ static int numberAreaWidth()
 // OutputPaneManager
 ////
 
-static OutputPaneManager *m_instance = 0;
+static OutputPaneManager *m_instance = nullptr;
 
 void OutputPaneManager::create()
 {
@@ -106,7 +128,7 @@ void OutputPaneManager::create()
 void OutputPaneManager::destroy()
 {
     delete m_instance;
-    m_instance = 0;
+    m_instance = nullptr;
 }
 
 OutputPaneManager *OutputPaneManager::instance()
@@ -119,9 +141,11 @@ void OutputPaneManager::updateStatusButtons(bool visible)
     int idx = currentIndex();
     if (idx == -1)
         return;
-    QTC_ASSERT(g_outputPanes.size() == m_buttons.size(), return);
-    m_buttons.at(idx)->setChecked(visible);
-    g_outputPanes.at(idx)->visibilityChanged(visible);
+    QTC_ASSERT(idx < g_outputPanes.size(), return);
+    const OutputPaneData &data = g_outputPanes.at(idx);
+    QTC_ASSERT(data.button, return);
+    data.button->setChecked(visible);
+    data.pane->visibilityChanged(visible);
 }
 
 void OutputPaneManager::updateMaximizeButton(bool maximized)
@@ -135,20 +159,26 @@ void OutputPaneManager::updateMaximizeButton(bool maximized)
     }
 }
 
+// Return shortcut as Alt+<number> or Cmd+<number> if number is a non-zero digit
+static QKeySequence paneShortCut(int number)
+{
+    if (number < 1 || number > 9)
+        return QKeySequence();
+
+    const int modifier = HostOsInfo::isMacHost() ? Qt::CTRL : Qt::ALT;
+    return QKeySequence(modifier | (Qt::Key_0 + number));
+}
+
 OutputPaneManager::OutputPaneManager(QWidget *parent) :
     QWidget(parent),
     m_titleLabel(new QLabel),
     m_manageButton(new OutputPaneManageButton),
     m_closeButton(new QToolButton),
-    m_minMaxAction(0),
     m_minMaxButton(new QToolButton),
-    m_nextAction(0),
-    m_prevAction(0),
     m_outputWidgetPane(new QStackedWidget),
     m_opToolBarWidgets(new QStackedWidget),
     m_minimizeIcon(Utils::Icons::ARROW_DOWN.icon()),
-    m_maximizeIcon(Utils::Icons::ARROW_UP.icon()),
-    m_outputPaneHeightSetting(0)
+    m_maximizeIcon(Utils::Icons::ARROW_UP.icon())
 {
     setWindowTitle(tr("Output"));
 
@@ -178,11 +208,11 @@ OutputPaneManager::OutputPaneManager(QWidget *parent) :
 
     connect(ICore::instance(), &ICore::saveSettingsRequested, this, &OutputPaneManager::saveSettings);
 
-    QVBoxLayout *mainlayout = new QVBoxLayout;
+    auto mainlayout = new QVBoxLayout;
     mainlayout->setSpacing(0);
     mainlayout->setMargin(0);
     m_toolBar = new StyledBar;
-    QHBoxLayout *toolLayout = new QHBoxLayout(m_toolBar);
+    auto toolLayout = new QHBoxLayout(m_toolBar);
     toolLayout->setMargin(0);
     toolLayout->setSpacing(0);
     toolLayout->addWidget(m_titleLabel);
@@ -202,33 +232,14 @@ OutputPaneManager::OutputPaneManager(QWidget *parent) :
     setLayout(mainlayout);
 
     m_buttonsWidget = new QWidget;
+    m_buttonsWidget->setObjectName("OutputPaneButtons"); // used for UI introduction
     m_buttonsWidget->setLayout(new QHBoxLayout);
     m_buttonsWidget->layout()->setContentsMargins(5,0,0,0);
     m_buttonsWidget->layout()->setSpacing(
             creatorTheme()->flag(Theme::FlatToolBars) ? 9 : 4);
-}
 
-OutputPaneManager::~OutputPaneManager()
-{
-}
+    StatusBarManager::addStatusBarWidget(m_buttonsWidget, StatusBarManager::Second);
 
-QWidget *OutputPaneManager::buttonsWidget()
-{
-    return m_buttonsWidget;
-}
-
-// Return shortcut as Alt+<number> or Cmd+<number> if number is a non-zero digit
-static inline QKeySequence paneShortCut(int number)
-{
-    if (number < 1 || number > 9)
-        return QKeySequence();
-
-    const int modifier = HostOsInfo::isMacHost() ? Qt::CTRL : Qt::ALT;
-    return QKeySequence(modifier | (Qt::Key_0 + number));
-}
-
-void OutputPaneManager::init()
-{
     ActionContainer *mwindow = ActionManager::actionContainer(Constants::M_WINDOW);
 
     // Window->Output Panes
@@ -258,7 +269,7 @@ void OutputPaneManager::init()
     mpanes->addAction(cmd, "Coreplugin.OutputPane.ActionsGroup");
 
     cmd = ActionManager::registerAction(m_minMaxAction, "Coreplugin.OutputPane.minmax");
-    cmd->setDefaultKeySequence(QKeySequence(UseMacShortcuts ? tr("Ctrl+Shift+9") : tr("Alt+Shift+9")));
+    cmd->setDefaultKeySequence(QKeySequence(useMacShortcuts ? tr("Ctrl+Shift+9") : tr("Alt+Shift+9")));
     cmd->setAttribute(Command::CA_UpdateText);
     cmd->setAttribute(Command::CA_UpdateIcon);
     mpanes->addAction(cmd, "Coreplugin.OutputPane.ActionsGroup");
@@ -270,30 +281,40 @@ void OutputPaneManager::init()
     QFontMetrics titleFm = m_titleLabel->fontMetrics();
     int minTitleWidth = 0;
 
-    Utils::sort(g_outputPanes, [](IOutputPane *p1, IOutputPane *p2) {
-        return p1->priorityInStatusBar() > p2->priorityInStatusBar();
+    Utils::sort(g_outputPanes, [](const OutputPaneData &d1, const OutputPaneData &d2) {
+        return d1.pane->priorityInStatusBar() > d2.pane->priorityInStatusBar();
     });
     const int n = g_outputPanes.size();
 
     int shortcutNumber = 1;
     const Id baseId = "QtCreator.Pane.";
     for (int i = 0; i != n; ++i) {
-        IOutputPane *outPane = g_outputPanes.at(i);
+        OutputPaneData &data = g_outputPanes[i];
+        IOutputPane *outPane = data.pane;
         const int idx = m_outputWidgetPane->addWidget(outPane->outputWidget(this));
         QTC_CHECK(idx == i);
 
-        connect(outPane, &IOutputPane::showPage, this, [this, outPane](int flags) {
-            showPage(findIndexForPage(outPane), flags);
+        connect(outPane, &IOutputPane::showPage, this, [this, idx](int flags) {
+            showPage(idx, flags);
         });
         connect(outPane, &IOutputPane::hidePage, this, &OutputPaneManager::slotHide);
-        connect(outPane, &IOutputPane::togglePage, this, &OutputPaneManager::togglePage);
-        connect(outPane, &IOutputPane::navigateStateUpdate,
-                this, &OutputPaneManager::updateNavigateState);
-        connect(outPane, &IOutputPane::flashButton, this, &OutputPaneManager::flashButton);
-        connect(outPane, &IOutputPane::setBadgeNumber, this, &OutputPaneManager::setBadgeNumber);
+
+        connect(outPane, &IOutputPane::togglePage, this, [this, idx](int flags) {
+            if (OutputPanePlaceHolder::isCurrentVisible() && currentIndex() == idx)
+                slotHide();
+            else
+                showPage(idx, flags);
+        });
+
+        connect(outPane, &IOutputPane::navigateStateUpdate, this, [this, idx, outPane] {
+            if (currentIndex() == idx) {
+                m_prevAction->setEnabled(outPane->canNavigate() && outPane->canPrevious());
+                m_nextAction->setEnabled(outPane->canNavigate() && outPane->canNext());
+            }
+        });
 
         QWidget *toolButtonsContainer = new QWidget(m_opToolBarWidgets);
-        QHBoxLayout *toolButtonsLayout = new QHBoxLayout;
+        auto toolButtonsLayout = new QHBoxLayout;
         toolButtonsLayout->setMargin(0);
         toolButtonsLayout->setSpacing(0);
         foreach (QWidget *toolButton, outPane->toolBarWidgets())
@@ -307,29 +328,34 @@ void OutputPaneManager::init()
 
         QString suffix = outPane->displayName().simplified();
         suffix.remove(QLatin1Char(' '));
-        const Id id = baseId.withSuffix(suffix);
-        QAction *action = new QAction(outPane->displayName(), this);
-        Command *cmd = ActionManager::registerAction(action, id);
+        data.id = baseId.withSuffix(suffix);
+        data.action = new QAction(outPane->displayName(), this);
+        Command *cmd = ActionManager::registerAction(data.action, data.id);
 
         mpanes->addAction(cmd, "Coreplugin.OutputPane.PanesGroup");
-        m_actions.append(action);
-        m_ids.append(id);
 
         cmd->setDefaultKeySequence(paneShortCut(shortcutNumber));
-        OutputPaneToggleButton *button = new OutputPaneToggleButton(shortcutNumber, outPane->displayName(),
-                                                                    cmd->action());
+        auto button = new OutputPaneToggleButton(shortcutNumber, outPane->displayName(),
+                                                 cmd->action());
+        data.button = button;
+
+        connect(outPane, &IOutputPane::flashButton, button, [button] { button->flash(); });
+        connect(outPane, &IOutputPane::setBadgeNumber,
+                button, &OutputPaneToggleButton::setIconBadgeNumber);
+
         ++shortcutNumber;
-        m_buttonsWidget->layout()->addWidget(button);
-        m_buttons.append(button);
-        connect(button, &QAbstractButton::clicked, this, [this, button]() {
-            buttonTriggered(m_buttons.indexOf(button));
+        m_buttonsWidget->layout()->addWidget(data.button);
+        connect(data.button, &QAbstractButton::clicked, this, [this, i] {
+            buttonTriggered(i);
          });
 
         bool visible = outPane->priorityInStatusBar() != -1;
-        button->setVisible(visible);
-        m_buttonVisibility.insert(id, visible);
+        data.button->setVisible(visible);
+        data.buttonVisible = visible;
 
-        connect(action, &QAction::triggered, this, &OutputPaneManager::shortcutTriggered);
+        connect(data.action, &QAction::triggered, this, [this, i] {
+            shortcutTriggered(i);
+        });
     }
 
     m_titleLabel->setMinimumWidth(minTitleWidth + m_titleLabel->contentsMargins().left()
@@ -340,13 +366,11 @@ void OutputPaneManager::init()
     readSettings();
 }
 
-void OutputPaneManager::shortcutTriggered()
+OutputPaneManager::~OutputPaneManager() = default;
+
+void OutputPaneManager::shortcutTriggered(int idx)
 {
-    QAction *action = qobject_cast<QAction*>(sender());
-    QTC_ASSERT(action, return);
-    int idx = m_actions.indexOf(action);
-    QTC_ASSERT(idx != -1, return);
-    IOutputPane *outputPane = g_outputPanes.at(idx);
+    IOutputPane *outputPane = g_outputPanes.at(idx).pane;
     // Now check the special case, the output window is already visible,
     // we are already on that page but the outputpane doesn't have focus
     // then just give it focus.
@@ -401,16 +425,16 @@ void OutputPaneManager::readSettings()
     int num = settings->beginReadArray(QLatin1String(outputPaneSettingsKeyC));
     for (int i = 0; i < num; ++i) {
         settings->setArrayIndex(i);
-        m_buttonVisibility.insert(Id::fromSetting(settings->value(QLatin1String(outputPaneIdKeyC))),
-                          settings->value(QLatin1String(outputPaneVisibleKeyC)).toBool());
+        Id id = Id::fromSetting(settings->value(QLatin1String(outputPaneIdKeyC)));
+        const int idx = Utils::indexOf(g_outputPanes, Utils::equal(&OutputPaneData::id, id));
+        if (idx < 0) // happens for e.g. disabled plugins (with outputpanes) that were loaded before
+            continue;
+        const bool visible = settings->value(QLatin1String(outputPaneVisibleKeyC)).toBool();
+        g_outputPanes[idx].buttonVisible = visible;
+        g_outputPanes[idx].button->setVisible(visible);
     }
     settings->endArray();
 
-    for (int i = 0; i < m_ids.size(); ++i) {
-        if (m_buttonVisibility.contains(m_ids.at(i)))
-            m_buttons.at(i)->setVisible(m_buttonVisibility.value(m_ids.at(i)));
-    }
-
     m_outputPaneHeightSetting = settings->value(QLatin1String("OutputPanePlaceHolder/Height"), 0).toInt();
 }
 
@@ -418,7 +442,7 @@ void OutputPaneManager::slotNext()
 {
     int idx = currentIndex();
     ensurePageVisible(idx);
-    IOutputPane *out = g_outputPanes.at(idx);
+    IOutputPane *out = g_outputPanes.at(idx).pane;
     if (out->canNext())
         out->goToNext();
 }
@@ -427,7 +451,7 @@ void OutputPaneManager::slotPrev()
 {
     int idx = currentIndex();
     ensurePageVisible(idx);
-    IOutputPane *out = g_outputPanes.at(idx);
+    IOutputPane *out = g_outputPanes.at(idx).pane;
     if (out->canPrevious())
         out->goToPrev();
 }
@@ -439,8 +463,8 @@ void OutputPaneManager::slotHide()
         ph->setVisible(false);
         int idx = currentIndex();
         QTC_ASSERT(idx >= 0, return);
-        m_buttons.at(idx)->setChecked(false);
-        g_outputPanes.value(idx)->visibilityChanged(false);
+        g_outputPanes.at(idx).button->setChecked(false);
+        g_outputPanes.at(idx).pane->visibilityChanged(false);
         if (IEditor *editor = EditorManager::currentEditor()) {
             QWidget *w = editor->widget()->focusWidget();
             if (!w)
@@ -450,11 +474,6 @@ void OutputPaneManager::slotHide()
     }
 }
 
-int OutputPaneManager::findIndexForPage(IOutputPane *out)
-{
-    return g_outputPanes.indexOf(out);
-}
-
 void OutputPaneManager::ensurePageVisible(int idx)
 {
     //int current = currentIndex();
@@ -463,32 +482,6 @@ void OutputPaneManager::ensurePageVisible(int idx)
     setCurrentIndex(idx);
 }
 
-void OutputPaneManager::updateNavigateState()
-{
-    IOutputPane *pane = qobject_cast<IOutputPane*>(sender());
-    int idx = findIndexForPage(pane);
-    if (currentIndex() == idx) {
-        m_prevAction->setEnabled(pane->canNavigate() && pane->canPrevious());
-        m_nextAction->setEnabled(pane->canNavigate() && pane->canNext());
-    }
-}
-
-void OutputPaneManager::flashButton()
-{
-    IOutputPane* pane = qobject_cast<IOutputPane*>(sender());
-    int idx = findIndexForPage(pane);
-    if (pane)
-        m_buttons.value(idx)->flash();
-}
-
-void OutputPaneManager::setBadgeNumber(int number)
-{
-    IOutputPane* pane = qobject_cast<IOutputPane*>(sender());
-    int idx = findIndexForPage(pane);
-    if (pane)
-        m_buttons.value(idx)->setIconBadgeNumber(number);
-}
-
 void OutputPaneManager::showPage(int idx, int flags)
 {
     QTC_ASSERT(idx >= 0, return);
@@ -502,19 +495,18 @@ void OutputPaneManager::showPage(int idx, int flags)
     }
 
     bool onlyFlash = !ph
-            || (g_outputPanes.at(currentIndex())->hasFocus()
+            || (g_outputPanes.at(currentIndex()).pane->hasFocus()
                 && !(flags & IOutputPane::WithFocus)
                 && idx != currentIndex());
 
     if (onlyFlash) {
-        m_buttons.value(idx)->flash();
+        g_outputPanes.at(idx).button->flash();
     } else {
         // make the page visible
         ph->setVisible(true);
 
         ensurePageVisible(idx);
-        IOutputPane *out = g_outputPanes.at(idx);
-        out->visibilityChanged(true);
+        IOutputPane *out = g_outputPanes.at(idx).pane;
         if (flags & IOutputPane::WithFocus) {
             if (out->canFocus())
                 out->setFocus();
@@ -526,15 +518,6 @@ void OutputPaneManager::showPage(int idx, int flags)
     }
 }
 
-void OutputPaneManager::togglePage(int flags)
-{
-    int idx = findIndexForPage(qobject_cast<IOutputPane*>(sender()));
-    if (OutputPanePlaceHolder::isCurrentVisible() && currentIndex() == idx)
-         slotHide();
-    else
-         showPage(idx, flags);
-}
-
 void OutputPaneManager::focusInEvent(QFocusEvent *e)
 {
     if (QWidget *w = m_outputWidgetPane->currentWidget())
@@ -546,21 +529,24 @@ void OutputPaneManager::setCurrentIndex(int idx)
     static int lastIndex = -1;
 
     if (lastIndex != -1) {
-        m_buttons.at(lastIndex)->setChecked(false);
-        g_outputPanes.at(lastIndex)->visibilityChanged(false);
+        g_outputPanes.at(lastIndex).button->setChecked(false);
+        g_outputPanes.at(lastIndex).pane->visibilityChanged(false);
     }
 
     if (idx != -1) {
         m_outputWidgetPane->setCurrentIndex(idx);
         m_opToolBarWidgets->setCurrentIndex(idx);
 
-        IOutputPane *pane = g_outputPanes.at(idx);
+        OutputPaneData &data = g_outputPanes[idx];
+        IOutputPane *pane = data.pane;
+        data.button->show();
+        data.buttonVisible = true;
         pane->visibilityChanged(true);
 
         bool canNavigate = pane->canNavigate();
         m_prevAction->setEnabled(canNavigate && pane->canPrevious());
         m_nextAction->setEnabled(canNavigate && pane->canNext());
-        m_buttons.at(idx)->setChecked(OutputPanePlaceHolder::isCurrentVisible());
+        g_outputPanes.at(idx).button->setChecked(OutputPanePlaceHolder::isCurrentVisible());
         m_titleLabel->setText(pane->displayName());
     }
 
@@ -571,10 +557,10 @@ void OutputPaneManager::popupMenu()
 {
     QMenu menu;
     int idx = 0;
-    for (IOutputPane *pane : g_outputPanes) {
-        QAction *act = menu.addAction(pane->displayName());
+    for (OutputPaneData &data : g_outputPanes) {
+        QAction *act = menu.addAction(data.pane->displayName());
         act->setCheckable(true);
-        act->setChecked(m_buttonVisibility.value(m_ids.at(idx)));
+        act->setChecked(data.buttonVisible);
         act->setData(idx);
         ++idx;
     }
@@ -582,17 +568,14 @@ void OutputPaneManager::popupMenu()
     if (!result)
         return;
     idx = result->data().toInt();
-    Id id = m_ids.at(idx);
-    QTC_ASSERT(idx >= 0 && idx < m_buttons.size(), return);
-    QToolButton *button = m_buttons.at(idx);
-    if (m_buttonVisibility.value(id)) {
-        g_outputPanes.value(idx)->visibilityChanged(false);
-        button->setChecked(false);
-        button->hide();
-        m_buttonVisibility.insert(id, false);
+    QTC_ASSERT(idx >= 0 && idx < g_outputPanes.size(), return);
+    OutputPaneData &data = g_outputPanes[idx];
+    if (data.buttonVisible) {
+        data.pane->visibilityChanged(false);
+        data.button->setChecked(false);
+        data.button->hide();
+        data.buttonVisible = false;
     } else {
-        button->show();
-        m_buttonVisibility.insert(id, true);
         showPage(idx, IOutputPane::ModeSwitch);
     }
 }
@@ -600,13 +583,13 @@ void OutputPaneManager::popupMenu()
 void OutputPaneManager::saveSettings() const
 {
     QSettings *settings = ICore::settings();
-    settings->beginWriteArray(QLatin1String(outputPaneSettingsKeyC),
-                              m_ids.size());
-    for (int i = 0; i < m_ids.size(); ++i) {
+    const int n = g_outputPanes.size();
+    settings->beginWriteArray(QLatin1String(outputPaneSettingsKeyC), n);
+    for (int i = 0; i < n; ++i) {
+        const OutputPaneData &data = g_outputPanes.at(i);
         settings->setArrayIndex(i);
-        settings->setValue(QLatin1String(outputPaneIdKeyC), m_ids.at(i).toSetting());
-        settings->setValue(QLatin1String(outputPaneVisibleKeyC),
-                           m_buttonVisibility.value(m_ids.at(i)));
+        settings->setValue(QLatin1String(outputPaneIdKeyC), data.id.toSetting());
+        settings->setValue(QLatin1String(outputPaneVisibleKeyC), data.buttonVisible);
     }
     settings->endArray();
     int heightSetting = m_outputPaneHeightSetting;
@@ -620,7 +603,7 @@ void OutputPaneManager::clearPage()
 {
     int idx = currentIndex();
     if (idx >= 0)
-        g_outputPanes.at(idx)->clearContents();
+        g_outputPanes.at(idx).pane->clearContents();
 }
 
 int OutputPaneManager::currentIndex() const
@@ -703,7 +686,7 @@ void OutputPaneToggleButton::paintEvent(QPaintEvent*)
         if (c != Theme::BackgroundColorDark)
             p.fillRect(rect(), creatorTheme()->color(c));
     } else {
-        const QImage *image = 0;
+        const QImage *image = nullptr;
         if (isDown()) {
             static const QImage pressed(
                         StyleHelper::dpiSpecificImageFile(":/utils/images/panel_button_pressed.png"));
