@@ -30,6 +30,7 @@
 #include <QSharedPointer>
 
 QT_BEGIN_NAMESPACE
+class QAuthenticator;
 class QNetworkReply;
 class QWidget;
 QT_END_NAMESPACE
@@ -62,7 +63,7 @@ public:
     virtual bool hasSettings() const;
     virtual Core::IOptionsPage *settingsPage() const;
 
-    virtual bool checkConfiguration(QString *errorMessage = 0);
+    virtual bool checkConfiguration(QString *errorMessage = nullptr);
     virtual void fetch(const QString &id) = 0;
     virtual void list();
     virtual void paste(const QString &text,
@@ -79,11 +80,11 @@ public:
     // Return true when settings changed.
     static bool showConfigurationError(const Protocol *p,
                                        const QString &message,
-                                       QWidget *parent = 0,
+                                       QWidget *parent = nullptr,
                                        bool showConfig = true);
     // Ensure configuration is correct
     static bool ensureConfiguration(Protocol *p,
-                                    QWidget *parent = 0);
+                                    QWidget *parent = nullptr);
 
 signals:
     void pasteDone(const QString &link);
@@ -107,12 +108,20 @@ class NetworkProtocol : public Protocol
     Q_OBJECT
 
 public:
+    NetworkProtocol();
+
     ~NetworkProtocol() override;
 
-protected:
-    QNetworkReply *httpGet(const QString &url);
+signals:
+    void authenticationRequired(QNetworkReply *reply, QAuthenticator *authenticator);
 
-    QNetworkReply *httpPost(const QString &link, const QByteArray &data);
+protected:
+    void requestAuthentication(const QUrl &url, QNetworkReply *reply, QAuthenticator *authenticator);
+
+    QNetworkReply *httpGet(const QString &url, bool handleCookies = false);
+
+    QNetworkReply *httpPost(const QString &link, const QByteArray &data,
+                            bool handleCookies = false);
 
     // Check connectivity of host, displaying a message box.
     bool httpStatus(QString url, QString *errorMessage, bool useHttps = false);
