@@ -26,18 +26,20 @@
 #include "debuggerdialogs.h"
 
 #include "debuggerkitinformation.h"
-#include "debuggerstartparameters.h"
 #include "debuggerruncontrol.h"
 #include "cdb/cdbengine.h"
 
 #include <coreplugin/icore.h>
 #include <projectexplorer/projectexplorerconstants.h>
-#include <projectexplorer/runnables.h>
 #include <projectexplorer/toolchain.h>
+
+#include <app/app_version.h>
 #include <utils/pathchooser.h>
 #include <utils/fancylineedit.h>
 #include <utils/qtcassert.h>
 
+#include <ssh/sshconnection.h>
+
 #include <QButtonGroup>
 #include <QCheckBox>
 #include <QComboBox>
@@ -71,8 +73,9 @@ class StartApplicationDialogPrivate
 public:
     KitChooser *kitChooser;
     QLabel *serverPortLabel;
-    QLabel *serverAddressLabel;
-    QLineEdit *serverAddressEdit;
+    QLabel *channelOverrideHintLabel;
+    QLabel *channelOverrideLabel;
+    QLineEdit *channelOverrideEdit;
     QSpinBox *serverPortSpinBox;
     PathChooser *localExecutablePathChooser;
     FancyLineEdit *arguments;
@@ -105,14 +108,16 @@ DebuggerKitChooser::DebuggerKitChooser(Mode mode, QWidget *parent)
     , m_hostAbi(Abi::hostAbi())
     , m_mode(mode)
 {
-    setKitMatcher([this](const Kit *k) {
+    setKitPredicate([this](const Kit *k) {
         // Match valid debuggers and restrict local debugging to compatible toolchains.
-        if (!DebuggerKitInformation::isValidDebugger(k))
+        auto errors = DebuggerKitInformation::configurationErrors(k);
+        // we do not care for mismatched ABI if we want *any* debugging
+        if (m_mode == AnyDebugging && errors == DebuggerKitInformation::DebuggerDoesNotMatch)
+            errors = DebuggerKitInformation::NoConfigurationError;
+        if (errors)
             return false;
-        if (m_mode == LocalDebugging) {
-            const ToolChain *tc = ToolChainKitInformation::toolChain(k);
-            return tc && tc->targetAbi().os() == m_hostAbi.os();
-        }
+        if (m_mode == LocalDebugging)
+            return ToolChainKitInformation::targetAbi(k).os() == m_hostAbi.os();
         return true;
     });
 }
@@ -142,8 +147,9 @@ public:
     Id kitId;
     uint serverPort;
     QString serverAddress;
-    StandardRunnable runnable;
+    Runnable runnable;
     bool breakAtMain = false;
+    bool runInTerminal = false;
     QString serverStartScript;
     QString debugInfoLocation;
 };
@@ -155,7 +161,7 @@ bool StartApplicationParameters::equals(const StartApplicationParameters &rhs) c
         && runnable.commandLineArguments == rhs.runnable.commandLineArguments
         && runnable.workingDirectory == rhs.runnable.workingDirectory
         && breakAtMain == rhs.breakAtMain
-        && runnable.runMode == rhs.runnable.runMode
+        && runInTerminal == rhs.runInTerminal
         && serverStartScript == rhs.serverStartScript
         && kitId == rhs.kitId
         && debugInfoLocation == rhs.debugInfoLocation
@@ -167,16 +173,16 @@ QString StartApplicationParameters::displayName() const
     const int maxLength = 60;
 
     QString name = FileName::fromString(runnable.executable).fileName()
-            + QLatin1Char(' ') + runnable.commandLineArguments;
+            + ' ' + runnable.commandLineArguments;
     if (name.size() > 60) {
-        int index = name.lastIndexOf(QLatin1Char(' '), maxLength);
+        int index = name.lastIndexOf(' ', maxLength);
         if (index == -1)
             index = maxLength;
         name.truncate(index);
-        name += QLatin1String("...");
+        name += "...";
     }
 
-    if (Kit *kit = KitManager::find(kitId))
+    if (Kit *kit = KitManager::kit(kitId))
         name += QString::fromLatin1(" (%1)").arg(kit->displayName());
 
     return name;
@@ -191,7 +197,7 @@ void StartApplicationParameters::toSettings(QSettings *settings) const
     settings->setValue("LastExternalExecutableArguments", runnable.commandLineArguments);
     settings->setValue("LastExternalWorkingDirectory", runnable.workingDirectory);
     settings->setValue("LastExternalBreakAtMain", breakAtMain);
-    settings->setValue("LastExternalRunInTerminal", runnable.runMode == ApplicationLauncher::Console);
+    settings->setValue("LastExternalRunInTerminal", runInTerminal);
     settings->setValue("LastServerStartScript", serverStartScript);
     settings->setValue("LastDebugInfoLocation", debugInfoLocation);
 }
@@ -205,8 +211,7 @@ void StartApplicationParameters::fromSettings(const QSettings *settings)
     runnable.commandLineArguments = settings->value("LastExternalExecutableArguments").toString();
     runnable.workingDirectory = settings->value("LastExternalWorkingDirectory").toString();
     breakAtMain = settings->value("LastExternalBreakAtMain").toBool();
-    runnable.runMode = settings->value("LastExternalRunInTerminal").toBool()
-            ? ApplicationLauncher::Console : ApplicationLauncher::Gui;
+    runInTerminal = settings->value("LastExternalRunInTerminal").toBool();
     serverStartScript = settings->value("LastServerStartScript").toString();
     debugInfoLocation = settings->value("LastDebugInfoLocation").toString();
 }
@@ -224,8 +229,8 @@ StartApplicationDialog::StartApplicationDialog(QWidget *parent)
     setWindowTitle(tr("Start Debugger"));
 
     d->kitChooser = new KitChooser(this);
-    d->kitChooser->setKitMatcher([this](const Kit *k) {
-        return DebuggerKitInformation::isValidDebugger(k);
+    d->kitChooser->setKitPredicate([](const Kit *k) {
+        return !DebuggerKitInformation::configurationErrors(k);
     });
     d->kitChooser->populate();
 
@@ -233,21 +238,30 @@ StartApplicationDialog::StartApplicationDialog(QWidget *parent)
     d->serverPortSpinBox = new QSpinBox(this);
     d->serverPortSpinBox->setRange(1, 65535);
 
-    d->serverAddressLabel = new QLabel(tr("Override server address"), this);
-    d->serverAddressEdit = new QLineEdit(this);
+    d->channelOverrideHintLabel =
+            new QLabel(tr("Normally, the running server is identified by the IP of the "
+                          "device in the kit and the server port selected above.\n"
+                          "You can choose another communication channel here, such as "
+                          "a serial line or custom ip:port."));
+
+    d->channelOverrideLabel = new QLabel(tr("Override server channel:"), this);
+    d->channelOverrideEdit = new QLineEdit(this);
+    //: "For example, /dev/ttyS0, COM1, 127.0.0.1:1234"
+    d->channelOverrideEdit->setPlaceholderText(
+        tr("For example, %1").arg("/dev/ttyS0, COM1, 127.0.0.1:1234"));
 
     d->localExecutablePathChooser = new PathChooser(this);
     d->localExecutablePathChooser->setExpectedKind(PathChooser::File);
     d->localExecutablePathChooser->setPromptDialogTitle(tr("Select Executable"));
-    d->localExecutablePathChooser->setHistoryCompleter(QLatin1String("LocalExecutable"));
+    d->localExecutablePathChooser->setHistoryCompleter("LocalExecutable");
 
     d->arguments = new FancyLineEdit(this);
-    d->arguments->setHistoryCompleter(QLatin1String("CommandlineArguments"));
+    d->arguments->setHistoryCompleter("CommandlineArguments");
 
     d->workingDirectory = new PathChooser(this);
     d->workingDirectory->setExpectedKind(PathChooser::ExistingDirectory);
     d->workingDirectory->setPromptDialogTitle(tr("Select Working Directory"));
-    d->workingDirectory->setHistoryCompleter(QLatin1String("WorkingDirectory"));
+    d->workingDirectory->setHistoryCompleter("WorkingDirectory");
 
     d->runInTerminalCheckBox = new QCheckBox(this);
 
@@ -259,7 +273,7 @@ StartApplicationDialog::StartApplicationDialog(QWidget *parent)
     d->serverStartScriptPathChooser->setPromptDialogTitle(tr("Select Server Start Script"));
     d->serverStartScriptPathChooser->setToolTip(tr(
         "This option can be used to point to a script that will be used "
-        "to start a debug server. If the field is empty, Qt Creator's "
+        "to start a debug server. If the field is empty, "
         "default methods to set up debug servers will be used."));
     d->serverStartScriptLabel = new QLabel(tr("&Server start script:"), this);
     d->serverStartScriptLabel->setBuddy(d->serverStartScriptPathChooser);
@@ -270,7 +284,7 @@ StartApplicationDialog::StartApplicationDialog(QWidget *parent)
     d->debuginfoPathChooser->setToolTip(tr(
         "Base path for external debug information and debug sources. "
         "If empty, $SYSROOT/usr/lib/debug will be chosen."));
-    d->debuginfoPathChooser->setHistoryCompleter(QLatin1String("Debugger.DebugLocation.History"));
+    d->debuginfoPathChooser->setHistoryCompleter("Debugger.DebugLocation.History");
 
     auto line = new QFrame(this);
     line->setFrameShape(QFrame::HLine);
@@ -290,7 +304,6 @@ StartApplicationDialog::StartApplicationDialog(QWidget *parent)
     formLayout->setFieldGrowthPolicy(QFormLayout::AllNonFixedFieldsGrow);
     formLayout->addRow(tr("&Kit:"), d->kitChooser);
     formLayout->addRow(d->serverPortLabel, d->serverPortSpinBox);
-    formLayout->addRow(d->serverAddressLabel, d->serverAddressEdit);
     formLayout->addRow(tr("Local &executable:"), d->localExecutablePathChooser);
     formLayout->addRow(tr("Command line &arguments:"), d->arguments);
     formLayout->addRow(tr("&Working directory:"), d->workingDirectory);
@@ -298,6 +311,8 @@ StartApplicationDialog::StartApplicationDialog(QWidget *parent)
     formLayout->addRow(tr("Break at \"&main\":"), d->breakAtMainCheckBox);
     formLayout->addRow(d->serverStartScriptLabel, d->serverStartScriptPathChooser);
     formLayout->addRow(tr("Debug &information:"), d->debuginfoPathChooser);
+    formLayout->addRow(d->channelOverrideHintLabel);
+    formLayout->addRow(d->channelOverrideLabel, d->channelOverrideEdit);
     formLayout->addRow(line2);
     formLayout->addRow(tr("&Recent:"), d->historyComboBox);
 
@@ -314,6 +329,9 @@ StartApplicationDialog::StartApplicationDialog(QWidget *parent)
     connect(d->historyComboBox, static_cast<void(QComboBox::*)(int)>(&QComboBox::currentIndexChanged),
             this, &StartApplicationDialog::historyIndexChanged);
 
+    connect(d->channelOverrideEdit, &QLineEdit::textChanged,
+            this, &StartApplicationDialog::onChannelOverrideChanged);
+
     updateState();
 }
 
@@ -332,6 +350,12 @@ void StartApplicationDialog::setHistory(const QList<StartApplicationParameters>
     }
 }
 
+void StartApplicationDialog::onChannelOverrideChanged(const QString &channel)
+{
+    d->serverPortSpinBox->setEnabled(channel.isEmpty());
+    d->serverPortLabel->setEnabled(channel.isEmpty());
+}
+
 void StartApplicationDialog::historyIndexChanged(int index)
 {
     if (index < 0)
@@ -347,11 +371,10 @@ void StartApplicationDialog::updateState()
     d->buttonBox->button(QDialogButtonBox::Ok)->setEnabled(okEnabled);
 }
 
-bool StartApplicationDialog::run(QWidget *parent, DebuggerRunParameters *rp, Kit **kit)
+void StartApplicationDialog::run(bool attachRemote)
 {
-    const bool attachRemote = rp->startMode == AttachToRemoteServer;
-    const QString settingsGroup = QLatin1String("DebugMode");
-    const QString arrayName = QLatin1String("StartApplication");
+    const QString settingsGroup = "DebugMode";
+    const QString arrayName = "StartApplication";
 
     QList<StartApplicationParameters> history;
     QSettings *settings = ICore::settings();
@@ -369,7 +392,7 @@ bool StartApplicationDialog::run(QWidget *parent, DebuggerRunParameters *rp, Kit
     settings->endArray();
     settings->endGroup();
 
-    StartApplicationDialog dialog(parent);
+    StartApplicationDialog dialog(ICore::dialogParent());
     dialog.setHistory(history);
     dialog.setParameters(history.back());
     if (!attachRemote) {
@@ -377,11 +400,18 @@ bool StartApplicationDialog::run(QWidget *parent, DebuggerRunParameters *rp, Kit
         dialog.d->serverStartScriptLabel->setVisible(false);
         dialog.d->serverPortSpinBox->setVisible(false);
         dialog.d->serverPortLabel->setVisible(false);
-        dialog.d->serverAddressLabel->setVisible(false);
-        dialog.d->serverAddressEdit->setVisible(false);
+        dialog.d->channelOverrideHintLabel->setVisible(false);
+        dialog.d->channelOverrideLabel->setVisible(false);
+        dialog.d->channelOverrideEdit->setVisible(false);
     }
     if (dialog.exec() != QDialog::Accepted)
-        return false;
+        return;
+
+    Kit *k = dialog.d->kitChooser->currentKit();
+    IDevice::ConstPtr dev = DeviceKitInformation::device(k);
+
+    auto runControl = new RunControl(nullptr, ProjectExplorer::Constants::DEBUG_RUN_MODE);
+    auto debugger = new DebuggerRunTool(runControl, k);
 
     const StartApplicationParameters newParameters = dialog.parameters();
     if (newParameters != history.back()) {
@@ -398,38 +428,51 @@ bool StartApplicationDialog::run(QWidget *parent, DebuggerRunParameters *rp, Kit
         settings->endGroup();
     }
 
-    rp->inferior.executable = newParameters.runnable.executable;
-    const QString inputAddress = dialog.d->serverAddressEdit->text();
+    Runnable inferior = newParameters.runnable;
+    const QString inputAddress = dialog.d->channelOverrideEdit->text();
     if (!inputAddress.isEmpty())
-        rp->remoteChannel = inputAddress;
+        debugger->setRemoteChannel(inputAddress);
     else
-        rp->remoteChannel = rp->connParams.host;
-    if (!rp->remoteChannel.isEmpty())
-        rp->remoteChannel += QLatin1Char(':') + QString::number(newParameters.serverPort);
-    rp->displayName = newParameters.displayName();
-    rp->inferior.workingDirectory = newParameters.runnable.workingDirectory;
-    rp->useTerminal = newParameters.runnable.runMode == ApplicationLauncher::Console;
-    if (!newParameters.runnable.commandLineArguments.isEmpty())
-        rp->inferior.commandLineArguments = newParameters.runnable.commandLineArguments;
-    rp->breakOnMain = newParameters.breakAtMain;
-    rp->serverStartScript = newParameters.serverStartScript;
-    rp->debugInfoLocation = newParameters.debugInfoLocation;
+        debugger->setRemoteChannel(dev->sshParameters().host(), newParameters.serverPort);
+    debugger->setRunControlName(newParameters.displayName());
+    debugger->setBreakOnMain(newParameters.breakAtMain);
+    debugger->setDebugInfoLocation(newParameters.debugInfoLocation);
+    debugger->setInferior(inferior);
+    debugger->setServerStartScript(newParameters.serverStartScript); // Note: This requires inferior.
+    debugger->setUseTerminal(newParameters.runInTerminal);
 
-    Kit *k = dialog.d->kitChooser->currentKit();
-    IDevice::ConstPtr dev = DeviceKitInformation::device(k);
     bool isLocal = !dev || (dev->type() == ProjectExplorer::Constants::DESKTOP_DEVICE_TYPE);
+    if (isLocal) {
+        Environment inferiorEnvironment = Environment::systemEnvironment();
+        k->addToEnvironment(inferiorEnvironment);
+        debugger->setInferiorEnvironment(inferiorEnvironment);
+    }
     if (!attachRemote)
-        rp->startMode = isLocal ? StartExternal : StartRemoteProcess;
-    if (kit)
-        *kit = k;
-    return true;
+        debugger->setStartMode(isLocal ? StartExternal : StartRemoteProcess);
+
+    if (attachRemote) {
+        debugger->setStartMode(AttachToRemoteServer);
+        debugger->setCloseMode(KillAtClose);
+        debugger->setUseContinueInsteadOfRun(true);
+    }
+    debugger->startRunControl();
+}
+
+void StartApplicationDialog::attachToRemoteServer()
+{
+    run(true);
+}
+
+void StartApplicationDialog::startAndDebugApplication()
+{
+    run(false);
 }
 
 StartApplicationParameters StartApplicationDialog::parameters() const
 {
     StartApplicationParameters result;
     result.serverPort = d->serverPortSpinBox->value();
-    result.serverAddress = d->serverAddressEdit->text();
+    result.serverAddress = d->channelOverrideEdit->text();
     result.runnable.executable = d->localExecutablePathChooser->path();
     result.serverStartScript = d->serverStartScriptPathChooser->path();
     result.kitId = d->kitChooser->currentKitId();
@@ -437,8 +480,7 @@ StartApplicationParameters StartApplicationDialog::parameters() const
     result.runnable.commandLineArguments = d->arguments->text();
     result.runnable.workingDirectory = d->workingDirectory->path();
     result.breakAtMain = d->breakAtMainCheckBox->isChecked();
-    result.runnable.runMode = d->runInTerminalCheckBox->isChecked()
-            ? ApplicationLauncher::Console : ApplicationLauncher::Gui;
+    result.runInTerminal = d->runInTerminalCheckBox->isChecked();
     return result;
 }
 
@@ -446,13 +488,13 @@ void StartApplicationDialog::setParameters(const StartApplicationParameters &p)
 {
     d->kitChooser->setCurrentKitId(p.kitId);
     d->serverPortSpinBox->setValue(p.serverPort);
-    d->serverAddressEdit->setText(p.serverAddress);
+    d->channelOverrideEdit->setText(p.serverAddress);
     d->localExecutablePathChooser->setPath(p.runnable.executable);
     d->serverStartScriptPathChooser->setPath(p.serverStartScript);
     d->debuginfoPathChooser->setPath(p.debugInfoLocation);
     d->arguments->setText(p.runnable.commandLineArguments);
     d->workingDirectory->setPath(p.runnable.workingDirectory);
-    d->runInTerminalCheckBox->setChecked(p.runnable.runMode == ApplicationLauncher::Console);
+    d->runInTerminalCheckBox->setChecked(p.runInTerminal);
     d->breakAtMainCheckBox->setChecked(p.breakAtMain);
     updateState();
 }
@@ -541,19 +583,20 @@ static QString cdbRemoteHelp()
     const QString ext32 = QDir::toNativeSeparators(CdbEngine::extensionLibraryName(false));
     const QString ext64 = QDir::toNativeSeparators(CdbEngine::extensionLibraryName(true));
     return  StartRemoteCdbDialog::tr(
-                "<html><body><p>The remote CDB needs to load the matching Qt Creator CDB extension "
-                "(<code>%1</code> or <code>%2</code>, respectively).</p><p>Copy it onto the remote machine and set the "
-                "environment variable <code>%3</code> to point to its folder.</p><p>"
-                "Launch the remote CDB as <code>%4 &lt;executable&gt;</code> "
+                "<html><body><p>The remote CDB needs to load the matching %1 CDB extension "
+                "(<code>%2</code> or <code>%3</code>, respectively).</p><p>Copy it onto the remote machine and set the "
+                "environment variable <code>%4</code> to point to its folder.</p><p>"
+                "Launch the remote CDB as <code>%5 &lt;executable&gt;</code> "
                 "to use TCP/IP as communication protocol.</p><p>Enter the connection parameters as:</p>"
-                "<pre>%5</pre></body></html>").
-            arg(ext32, ext64, QLatin1String("_NT_DEBUGGER_EXTENSION_PATH"),
-                QLatin1String("cdb.exe -server tcp:port=1234"),
-                QLatin1String(cdbConnectionSyntax));
+                "<pre>%6</pre></body></html>")
+            .arg(Core::Constants::IDE_DISPLAY_NAME,
+                 ext32, ext64, "_NT_DEBUGGER_EXTENSION_PATH",
+                 "cdb.exe -server tcp:port=1234",
+                 QLatin1String(cdbConnectionSyntax));
 }
 
 StartRemoteCdbDialog::StartRemoteCdbDialog(QWidget *parent) :
-    QDialog(parent), m_okButton(0), m_lineEdit(new QLineEdit)
+    QDialog(parent), m_lineEdit(new QLineEdit)
 {
     setWindowTitle(tr("Start a CDB Remote Session"));
     setWindowFlags(windowFlags() & ~Qt::WindowContextHelpButtonHint);
@@ -598,9 +641,7 @@ void StartRemoteCdbDialog::accept()
         QDialog::accept();
 }
 
-StartRemoteCdbDialog::~StartRemoteCdbDialog()
-{
-}
+StartRemoteCdbDialog::~StartRemoteCdbDialog() = default;
 
 void StartRemoteCdbDialog::textChanged(const QString &t)
 {
@@ -611,7 +652,7 @@ QString StartRemoteCdbDialog::connection() const
 {
     const QString rc = m_lineEdit->text();
     // Transform an IP:POrt ('localhost:1234') specification into full spec
-    QRegExp ipRegexp(QLatin1String("([\\w\\.\\-_]+):([0-9]{1,4})"));
+    QRegExp ipRegexp("([\\w\\.\\-_]+):([0-9]{1,4})");
     QTC_ASSERT(ipRegexp.isValid(), return QString());
     if (ipRegexp.exactMatch(rc))
         return QString::fromLatin1("tcp:server=%1,port=%2").arg(ipRegexp.cap(1), ipRegexp.cap(2));
@@ -633,7 +674,7 @@ AddressDialog::AddressDialog(QWidget *parent) :
     setWindowFlags(windowFlags() & ~Qt::WindowContextHelpButtonHint);
 
     auto hLayout = new QHBoxLayout;
-    hLayout->addWidget(new QLabel(tr("Enter an address:") + QLatin1Char(' ')));
+    hLayout->addWidget(new QLabel(tr("Enter an address:") + ' '));
     hLayout->addWidget(m_lineEdit);
 
     auto vLayout = new QVBoxLayout;
@@ -661,12 +702,12 @@ bool AddressDialog::isOkButtonEnabled() const
 
 void AddressDialog::setAddress(quint64 a)
 {
-    m_lineEdit->setText(QLatin1String("0x") + QString::number(a, 16));
+    m_lineEdit->setText("0x" + QString::number(a, 16));
 }
 
 quint64 AddressDialog::address() const
 {
-    return m_lineEdit->text().toULongLong(0, 16);
+    return m_lineEdit->text().toULongLong(nullptr, 16);
 }
 
 void AddressDialog::accept()
@@ -712,19 +753,19 @@ StartRemoteEngineDialog::StartRemoteEngineDialog(QWidget *parent)
     setWindowTitle(tr("Start Remote Engine"));
 
     d->host = new FancyLineEdit(this);
-    d->host->setHistoryCompleter(QLatin1String("HostName"));
+    d->host->setHistoryCompleter("HostName");
 
     d->username = new FancyLineEdit(this);
-    d->username->setHistoryCompleter(QLatin1String("UserName"));
+    d->username->setHistoryCompleter("UserName");
 
     d->password = new QLineEdit(this);
     d->password->setEchoMode(QLineEdit::Password);
 
     d->enginePath = new FancyLineEdit(this);
-    d->enginePath->setHistoryCompleter(QLatin1String("EnginePath"));
+    d->enginePath->setHistoryCompleter("EnginePath");
 
     d->inferiorPath = new FancyLineEdit(this);
-    d->inferiorPath->setHistoryCompleter(QLatin1String("InferiorPath"));
+    d->inferiorPath->setHistoryCompleter("InferiorPath");
 
     d->buttonBox = new QDialogButtonBox(this);
     d->buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);
@@ -788,7 +829,7 @@ public:
     {
         m_layout = new QGridLayout;
         m_layout->setColumnStretch(0, 2);
-        QVBoxLayout *vboxLayout = new QVBoxLayout;
+        auto vboxLayout = new QVBoxLayout;
         vboxLayout->addLayout(m_layout);
         vboxLayout->addItem(new QSpacerItem(1, 1, QSizePolicy::Ignored,
                                             QSizePolicy::MinimumExpanding));
@@ -800,10 +841,10 @@ public:
     {
         const int row = m_layout->rowCount();
         int column = 0;
-        QButtonGroup *group = new QButtonGroup(this);
+        auto group = new QButtonGroup(this);
         m_layout->addWidget(new QLabel(type), row, column++);
         for (int i = -1; i != typeFormats.size(); ++i) {
-            QRadioButton *choice = new QRadioButton(this);
+            auto choice = new QRadioButton(this);
             choice->setText(i == -1 ? TypeFormatsDialog::tr("Reset")
                                     : WatchHandler::nameForFormat(typeFormats.at(i)));
             m_layout->addWidget(choice, row, column++);
@@ -826,7 +867,7 @@ public:
         buttonBox = new QDialogButtonBox(q);
         buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);
 
-        QVBoxLayout *layout = new QVBoxLayout(q);
+        auto layout = new QVBoxLayout(q);
         layout->addWidget(tabs);
         layout->addWidget(buttonBox);
         q->setLayout(layout);
@@ -834,9 +875,9 @@ public:
 
     void addPage(const QString &name)
     {
-        TypeFormatsDialogPage *page = new TypeFormatsDialogPage;
+        auto page = new TypeFormatsDialogPage;
         pages.append(page);
-        QScrollArea *scroller = new QScrollArea;
+        auto scroller = new QScrollArea;
         scroller->setWidgetResizable(true);
         scroller->setWidget(page);
         scroller->setFrameStyle(QFrame::NoFrame);
@@ -880,11 +921,11 @@ void TypeFormatsDialog::addTypeFormats(const QString &type0,
     const DisplayFormats &typeFormats, int current)
 {
     QString type = type0;
-    type.replace(QLatin1String("__"), QLatin1String("::"));
+    type.replace("__", "::");
     int pos = 2;
-    if (type.startsWith(QLatin1Char('Q')))
+    if (type.startsWith('Q'))
         pos = 0;
-    else if (type.startsWith(QLatin1String("std::")))
+    else if (type.startsWith("std::"))
         pos = 1;
     m_ui->pages[pos]->addTypeFormats(type, typeFormats, current);
 }
