@@ -73,32 +73,29 @@ QbsInstallStep::QbsInstallStep(ProjectExplorer::BuildStepList *bsl) :
 
 QbsInstallStep::~QbsInstallStep()
 {
-    cancel();
+    doCancel();
     if (m_job)
         m_job->deleteLater();
-    m_job = 0;
+    m_job = nullptr;
 }
 
-bool QbsInstallStep::init(QList<const BuildStep *> &earlierSteps)
+bool QbsInstallStep::init()
 {
-    Q_UNUSED(earlierSteps);
     QTC_ASSERT(!project()->isParsing() && !m_job, return false);
     return true;
 }
 
-void QbsInstallStep::run(QFutureInterface<bool> &fi)
+void QbsInstallStep::doRun()
 {
-    m_fi = &fi;
-
-    QbsProject *pro = static_cast<QbsProject *>(project());
+    auto pro = static_cast<QbsProject *>(project());
     m_job = pro->install(m_qbsInstallOptions);
 
     if (!m_job) {
-        reportRunResult(*m_fi, false);
+        emit finished(false);
         return;
     }
 
-    m_progressBase = 0;
+    m_maxProgress = 0;
 
     connect(m_job, &qbs::AbstractJob::finished, this, &QbsInstallStep::installDone);
     connect(m_job, &qbs::AbstractJob::taskStarted,
@@ -112,12 +109,7 @@ ProjectExplorer::BuildStepConfigWidget *QbsInstallStep::createConfigWidget()
     return new QbsInstallStepConfigWidget(this);
 }
 
-bool QbsInstallStep::runInGuiThread() const
-{
-    return true;
-}
-
-void QbsInstallStep::cancel()
+void QbsInstallStep::doCancel()
 {
     if (m_job)
         m_job->cancel();
@@ -146,8 +138,7 @@ bool QbsInstallStep::keepGoing() const
 
 const QbsBuildConfiguration *QbsInstallStep::buildConfig() const
 {
-    return static_cast<QbsBuildConfiguration *>(
-                deployConfiguration()->target()->activeBuildConfiguration());
+    return static_cast<QbsBuildConfiguration *>(buildConfiguration());
 }
 
 bool QbsInstallStep::fromMap(const QVariantMap &map)
@@ -186,25 +177,21 @@ void QbsInstallStep::installDone(bool success)
                             item.codeLocation().filePath(), item.codeLocation().line());
     }
 
-    QTC_ASSERT(m_fi, return);
-    reportRunResult(*m_fi, success);
-    m_fi = 0; // do not delete, it is not ours
+    emit finished(success);
     m_job->deleteLater();
-    m_job = 0;
+    m_job = nullptr;
 }
 
 void QbsInstallStep::handleTaskStarted(const QString &desciption, int max)
 {
-    Q_UNUSED(desciption);
-    QTC_ASSERT(m_fi, return);
-    m_progressBase = m_fi->progressValue();
-    m_fi->setProgressRange(0, m_progressBase + max);
+    m_description = desciption;
+    m_maxProgress = max;
 }
 
 void QbsInstallStep::handleProgress(int value)
 {
-    QTC_ASSERT(m_fi, return);
-    m_fi->setProgressValue(m_progressBase + value);
+    if (m_maxProgress > 0)
+        emit progress(value * 100 / m_maxProgress, m_description);
 }
 
 void QbsInstallStep::createTaskAndOutput(ProjectExplorer::Task::TaskType type,
@@ -252,7 +239,7 @@ void QbsInstallStep::handleBuildConfigChanged()
 // --------------------------------------------------------------------
 
 QbsInstallStepConfigWidget::QbsInstallStepConfigWidget(QbsInstallStep *step) :
-    m_step(step), m_ignoreChange(false)
+    BuildStepConfigWidget(step), m_step(step), m_ignoreChange(false)
 {
     connect(m_step, &ProjectExplorer::ProjectConfiguration::displayNameChanged,
             this, &QbsInstallStepConfigWidget::updateState);
@@ -261,7 +248,7 @@ QbsInstallStepConfigWidget::QbsInstallStepConfigWidget(QbsInstallStep *step) :
 
     setContentsMargins(0, 0, 0, 0);
 
-    QbsProject *project = static_cast<QbsProject *>(m_step->project());
+    auto project = static_cast<QbsProject *>(m_step->project());
 
     m_ui = new Ui::QbsInstallStepConfigWidget;
     m_ui->setupUi(this);
@@ -284,16 +271,6 @@ QbsInstallStepConfigWidget::~QbsInstallStepConfigWidget()
     delete m_ui;
 }
 
-QString QbsInstallStepConfigWidget::summaryText() const
-{
-    return m_summary;
-}
-
-QString QbsInstallStepConfigWidget::displayName() const
-{
-    return m_step->displayName();
-}
-
 void QbsInstallStepConfigWidget::updateState()
 {
     if (!m_ignoreChange) {
@@ -307,11 +284,7 @@ void QbsInstallStepConfigWidget::updateState()
 
     m_ui->commandLineTextEdit->setPlainText(command);
 
-    QString summary = tr("<b>Qbs:</b> %1").arg(command);
-    if (m_summary != summary) {
-        m_summary = summary;
-        emit updateSummary();
-    }
+    setSummaryText(tr("<b>Qbs:</b> %1").arg(command));
 }
 
 void QbsInstallStepConfigWidget::changeRemoveFirst(bool rf)
@@ -339,7 +312,7 @@ QbsInstallStepFactory::QbsInstallStepFactory()
     setSupportedStepList(ProjectExplorer::Constants::BUILDSTEPS_DEPLOY);
     setSupportedDeviceType(ProjectExplorer::Constants::DESKTOP_DEVICE_TYPE);
     setSupportedProjectType(Constants::PROJECT_ID);
-    setDisplayName(tr("Qbs Install"));
+    setDisplayName(QbsInstallStep::tr("Qbs Install"));
 }
 
 } // namespace Internal
