@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,22 +9,17 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://www.qt.io/licensing.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
@@ -41,7 +36,7 @@
 #include <viewmanager.h>
 #include <nodeinstanceview.h>
 
-#include <projectexplorer/projectexplorer.h>
+#include <projectexplorer/projecttree.h>
 #include <projectexplorer/project.h>
 #include <projectexplorer/target.h>
 #include <projectexplorer/session.h>
@@ -49,6 +44,10 @@
 #include <qtsupport/qtkitinformation.h>
 #include <qtsupport/qtsupportconstants.h>
 #include <qtsupport/qtversionmanager.h>
+#include <coreplugin/idocument.h>
+#include <coreplugin/editormanager/editormanager.h>
+
+#include <qmljs/qmljsmodelmanagerinterface.h>
 
 #include <QFileInfo>
 #include <QUrl>
@@ -57,6 +56,8 @@
 #include <QPlainTextEdit>
 #include <QApplication>
 
+using namespace ProjectExplorer;
+
 enum {
     debug = false
 };
@@ -76,13 +77,11 @@ DesignDocument::DesignDocument(QObject *parent) :
         m_subComponentManager(new SubComponentManager(m_documentModel.data(), this)),
         m_rewriterView (new RewriterView(RewriterView::Amend, m_documentModel.data())),
         m_documentLoaded(false),
-        m_currentKit(0)
+        m_currentKit(nullptr)
 {
 }
 
-DesignDocument::~DesignDocument()
-{
-}
+DesignDocument::~DesignDocument() = default;
 
 Model *DesignDocument::currentModel() const
 {
@@ -117,7 +116,7 @@ static ComponentTextModifier *createComponentTextModifier(TextModifier *original
                                                           const QString &componentText,
                                                           const ModelNode &componentNode)
 {
-    bool explicitComponent = componentText.contains("Component");
+    bool explicitComponent = componentText.contains(QLatin1String("Component"));
 
     ModelNode rootModelNode = rewriterView->rootModelNode();
 
@@ -134,12 +133,12 @@ static ComponentTextModifier *createComponentTextModifier(TextModifier *original
         componentEndOffset = componentStartOffset + rewriterView->nodeLength(componentNode);
     }
 
-    return new ComponentTextModifier (originalModifier, componentStartOffset, componentEndOffset, rootStartOffset);
+    return new ComponentTextModifier(originalModifier, componentStartOffset, componentEndOffset, rootStartOffset);
 }
 
 bool DesignDocument::loadInFileComponent(const ModelNode &componentNode)
 {
-    QString componentText = rewriterView()->extractText(QList<ModelNode>() << componentNode).value(componentNode);
+    QString componentText = rewriterView()->extractText({componentNode}).value(componentNode);
 
     if (componentText.isEmpty())
         return false;
@@ -165,22 +164,29 @@ Model* DesignDocument::createInFileComponentModel()
     return model;
 }
 
-/*!
-  Returns any errors that happened when parsing the latest qml file.
-  */
-QList<RewriterView::Error> DesignDocument::qmlSyntaxErrors() const
+QList<DocumentMessage> DesignDocument::qmlParseWarnings() const
+{
+    return m_rewriterView->warnings();
+}
+
+bool DesignDocument::hasQmlParseWarnings() const
+{
+    return !m_rewriterView->warnings().isEmpty();
+}
+
+QList<DocumentMessage> DesignDocument::qmlParseErrors() const
 {
     return m_rewriterView->errors();
 }
 
-bool DesignDocument::hasQmlSyntaxErrors() const
+bool DesignDocument::hasQmlParseErrors() const
 {
-    return currentModel()->rewriterView() && !currentModel()->rewriterView()->errors().isEmpty();
+    return !m_rewriterView->errors().isEmpty();
 }
 
 QString DesignDocument::displayName() const
 {
-    return fileName();
+    return fileName().toString();
 }
 
 QString DesignDocument::simplfiedDisplayName() const
@@ -189,30 +195,29 @@ QString DesignDocument::simplfiedDisplayName() const
         return rootModelNode().id();
     else
         return rootModelNode().simplifiedTypeName();
-
-    QStringList list = displayName().split(QLatin1Char('/'));
-    return list.last();
 }
 
-void DesignDocument::updateFileName(const QString & /*oldFileName*/, const QString &newFileName)
+void DesignDocument::updateFileName(const Utils::FileName & /*oldFileName*/, const Utils::FileName &newFileName)
 {
     if (m_documentModel)
-        m_documentModel->setFileUrl(QUrl::fromLocalFile(newFileName));
+        m_documentModel->setFileUrl(QUrl::fromLocalFile(newFileName.toString()));
 
     if (m_inFileComponentModel)
-        m_inFileComponentModel->setFileUrl(QUrl::fromLocalFile(newFileName));
+        m_inFileComponentModel->setFileUrl(QUrl::fromLocalFile(newFileName.toString()));
 
-    viewManager().setItemLibraryViewResourcePath(QFileInfo(newFileName).absolutePath());
+    viewManager().setItemLibraryViewResourcePath(newFileName.toFileInfo().absolutePath());
 
     emit displayNameChanged(displayName());
 }
 
-QString DesignDocument::fileName() const
+Utils::FileName DesignDocument::fileName() const
 {
-    return editor()->document()->filePath();
+    if (editor())
+        return editor()->document()->filePath();
+    return Utils::FileName();
 }
 
-ProjectExplorer::Kit *DesignDocument::currentKit() const
+Kit *DesignDocument::currentKit() const
 {
     return m_currentKit;
 }
@@ -231,19 +236,24 @@ void DesignDocument::loadDocument(QPlainTextEdit *edit)
 {
     Q_CHECK_PTR(edit);
 
-    connect(edit, SIGNAL(undoAvailable(bool)),
-            this, SIGNAL(undoAvailable(bool)));
-    connect(edit, SIGNAL(redoAvailable(bool)),
-            this, SIGNAL(redoAvailable(bool)));
-    connect(edit, SIGNAL(modificationChanged(bool)),
-            this, SIGNAL(dirtyStateChanged(bool)));
+    connect(edit, &QPlainTextEdit::undoAvailable,
+            this, &DesignDocument::undoAvailable);
+    connect(edit, &QPlainTextEdit::redoAvailable,
+            this, &DesignDocument::redoAvailable);
+    connect(edit, &QPlainTextEdit::modificationChanged,
+            this, &DesignDocument::dirtyStateChanged);
+
+    m_documentTextModifier.reset(new BaseTextEditModifier(dynamic_cast<TextEditor::TextEditorWidget*>(plainTextEdit())));
+
+    connect(m_documentTextModifier.data(), &TextModifier::textChanged, this, &DesignDocument::updateQrcFiles);
 
-    m_documentTextModifier.reset(new BaseTextEditModifier(dynamic_cast<TextEditor::BaseTextEditorWidget*>(plainTextEdit())));
     m_documentModel->setTextModifier(m_documentTextModifier.data());
 
     m_inFileComponentTextModifier.reset();
 
-    updateFileName(QString(), fileName());
+    updateFileName(Utils::FileName(), fileName());
+
+    updateQrcFiles();
 
     m_documentLoaded = true;
 }
@@ -273,6 +283,18 @@ void DesignDocument::changeToInFileComponentModel(ComponentTextModifier *textMod
     viewManager().attachViewsExceptRewriterAndComponetView();
 }
 
+void DesignDocument::updateQrcFiles()
+{
+    ProjectExplorer::Project *currentProject = ProjectExplorer::SessionManager::projectForFile(fileName());
+
+    if (currentProject) {
+        for (const Utils::FileName &fileName : currentProject->files(ProjectExplorer::Project::SourceFiles)) {
+            if (fileName.endsWith(".qrc"))
+                QmlJS::ModelManagerInterface::instance()->updateQrcFile(fileName.toString());
+        }
+    }
+}
+
 void DesignDocument::changeToSubComponent(const ModelNode &componentNode)
 {
     if (QmlDesignerPlugin::instance()->currentDesignDocument() != this)
@@ -336,7 +358,7 @@ void DesignDocument::close()
 void DesignDocument::updateSubcomponentManager()
 {
     Q_ASSERT(m_subComponentManager);
-    m_subComponentManager->update(QUrl::fromLocalFile(fileName()), currentModel()->imports());
+    m_subComponentManager->update(QUrl::fromLocalFile(fileName().toString()), currentModel()->imports());
 }
 
 void DesignDocument::deleteSelected()
@@ -352,67 +374,19 @@ void DesignDocument::deleteSelected()
                 QmlObjectNode(node).destroy();
         }
 
-    } catch (RewritingException &e) {
+        transaction.commit();
+    } catch (const RewritingException &e) {
         e.showException();
     }
 }
 
 void DesignDocument::copySelected()
 {
-    QScopedPointer<Model> copyModel(Model::create("QtQuick.Rectangle", 1, 0, currentModel()));
-    copyModel->setFileUrl(currentModel()->fileUrl());
-    copyModel->changeImports(currentModel()->imports(), QList<Import>());
-
-    Q_ASSERT(copyModel);
-
     DesignDocumentView view;
 
     currentModel()->attachView(&view);
 
-    if (view.selectedModelNodes().isEmpty())
-        return;
-
-    QList<ModelNode> selectedNodes(view.selectedModelNodes());
-
-    foreach (const ModelNode &node, selectedNodes) {
-        foreach (const ModelNode &node2, selectedNodes) {
-            if (node.isAncestorOf(node2))
-                selectedNodes.removeAll(node2);
-        }
-    }
-
-    if (selectedNodes.count() == 1) {
-        ModelNode selectedNode(selectedNodes.first());
-
-        if (!selectedNode.isValid())
-            return;
-
-        currentModel()->detachView(&view);
-
-        copyModel->attachView(&view);
-        view.replaceModel(selectedNode);
-
-        Q_ASSERT(view.rootModelNode().isValid());
-        Q_ASSERT(view.rootModelNode().type() != "empty");
-
-        view.toClipboard();
-    } else { //multi items selected
-        currentModel()->detachView(&view);
-        copyModel->attachView(&view);
-
-        foreach (ModelNode node, view.rootModelNode().directSubModelNodes()) {
-            node.destroy();
-        }
-        view.changeRootNodeType("QtQuick.Rectangle", 1, 0);
-        view.rootModelNode().setIdWithRefactoring("designer__Selection");
-
-        foreach (const ModelNode &selectedNode, selectedNodes) {
-            ModelNode newNode(view.insertModel(selectedNode));
-            view.rootModelNode().nodeListProperty("data").reparentHere(newNode);
-        }
-
-        view.toClipboard();
-    }
+    DesignDocumentView::copyModelNodes(view.selectedModelNodes());
 }
 
 void DesignDocument::cutSelected()
@@ -456,34 +430,27 @@ static void scatterItem(const ModelNode &pastedNode, const ModelNode &targetNode
 
 void DesignDocument::paste()
 {
-    QScopedPointer<Model> pasteModel(Model::create("empty", 1, 0, currentModel()));
-    pasteModel->setFileUrl(currentModel()->fileUrl());
-    pasteModel->changeImports(currentModel()->imports(), QList<Import>());
-
-    Q_ASSERT(pasteModel);
+    QScopedPointer<Model> pasteModel(DesignDocumentView::pasteToModel());
 
     if (!pasteModel)
         return;
 
     DesignDocumentView view;
     pasteModel->attachView(&view);
-
-    view.fromClipboard();
-
     ModelNode rootNode(view.rootModelNode());
+    QList<ModelNode> selectedNodes = rootNode.directSubModelNodes();
+    pasteModel->detachView(&view);
 
     if (rootNode.type() == "empty")
         return;
 
     if (rootNode.id() == "designer__Selection") {
-        QList<ModelNode> selectedNodes = rootNode.directSubModelNodes();
-        pasteModel->detachView(&view);
         currentModel()->attachView(&view);
 
         ModelNode targetNode;
 
         if (!view.selectedModelNodes().isEmpty())
-            targetNode = view.selectedModelNodes().first();
+            targetNode = view.selectedModelNodes().constFirst();
 
         //In case we copy and paste a selection we paste in the parent item
         if ((view.selectedModelNodes().count() == selectedNodes.count()) && targetNode.isValid() && targetNode.hasParentProperty())
@@ -515,20 +482,20 @@ void DesignDocument::paste()
             }
 
             view.setSelectedModelNodes(pastedNodeList);
-        } catch (RewritingException &e) {
+            transaction.commit();
+        } catch (const RewritingException &e) {
             qWarning() << e.description(); //silent error
         }
     } else {
         try {
             RewriterTransaction transaction(rewriterView(), QByteArrayLiteral("DesignDocument::paste2"));
 
-            pasteModel->detachView(&view);
             currentModel()->attachView(&view);
             ModelNode pastedNode(view.insertModel(rootNode));
             ModelNode targetNode;
 
             if (!view.selectedModelNodes().isEmpty())
-                targetNode = view.selectedModelNodes().first();
+                targetNode = view.selectedModelNodes().constFirst();
 
             if (!targetNode.isValid())
                 targetNode = view.rootModelNode();
@@ -552,8 +519,9 @@ void DesignDocument::paste()
             transaction.commit();
             NodeMetaInfo::clearCache();
 
-            view.setSelectedModelNodes(QList<ModelNode>() << pastedNode);
-        } catch (RewritingException &e) {
+            view.setSelectedModelNodes({pastedNode});
+            transaction.commit();
+        } catch (const RewritingException &e) {
             qWarning() << e.description(); //silent error
         }
     }
@@ -581,11 +549,28 @@ RewriterView *DesignDocument::rewriterView() const
 void DesignDocument::setEditor(Core::IEditor *editor)
 {
     m_textEditor = editor;
-    connect(editor->document(),
-            SIGNAL(filePathChanged(QString,QString)),
-            SLOT(updateFileName(QString,QString)));
+    // if the user closed the file explicit we do not want to do anything with it anymore
+
+
+    connect(Core::EditorManager::instance(), &Core::EditorManager::aboutToSave,
+            this, [this](Core::IDocument *document) {
+        if (m_textEditor && m_textEditor->document() == document) {
+            if (m_documentModel && m_documentModel->rewriterView())
+                m_documentModel->rewriterView()->writeAuxiliaryData();
+        }
+    });
+
+    connect(Core::EditorManager::instance(), &Core::EditorManager::editorAboutToClose,
+            this, [this](Core::IEditor *editor) {
+        if (m_textEditor.data() == editor)
+            m_textEditor.clear();
+    });
+
+    connect(editor->document(), &Core::IDocument::filePathChanged,
+            this, &DesignDocument::updateFileName);
 
     updateActiveQtVersion();
+    updateCurrentProject();
 }
 
 Core::IEditor *DesignDocument::editor() const
@@ -593,9 +578,9 @@ Core::IEditor *DesignDocument::editor() const
     return m_textEditor.data();
 }
 
-TextEditor::ITextEditor *DesignDocument::textEditor() const
+TextEditor::BaseTextEditor *DesignDocument::textEditor() const
 {
-    return qobject_cast<TextEditor::ITextEditor*>(editor());
+    return qobject_cast<TextEditor::BaseTextEditor*>(editor());
 }
 
 QPlainTextEdit *DesignDocument::plainTextEdit() const
@@ -603,7 +588,7 @@ QPlainTextEdit *DesignDocument::plainTextEdit() const
     if (editor())
         return qobject_cast<QPlainTextEdit*>(editor()->widget());
 
-    return 0;
+    return nullptr;
 }
 
 ModelNode DesignDocument::rootModelNode() const
@@ -627,41 +612,39 @@ void DesignDocument::redo()
     viewManager().resetPropertyEditorView();
 }
 
-static bool isFileInProject(DesignDocument *designDocument, ProjectExplorer::Project *project)
+static inline Kit *getActiveKit(DesignDocument *designDocument)
 {
-    foreach (const QString &fileNameInProject, project->files(ProjectExplorer::Project::ExcludeGeneratedFiles)) {
-        if (designDocument->fileName() == fileNameInProject)
-            return true;
-    }
-
-    return false;
-}
-
-static inline ProjectExplorer::Kit *getActiveKit(DesignDocument *designDocument)
-{
-    ProjectExplorer::ProjectExplorerPlugin *projectExplorer = ProjectExplorer::ProjectExplorerPlugin::instance();
-    ProjectExplorer::Project *currentProject = projectExplorer->currentProject();
+    ProjectExplorer::Project *currentProject = ProjectExplorer::SessionManager::projectForFile(designDocument->fileName());
 
     if (!currentProject)
-        currentProject = ProjectExplorer::SessionManager::projectForFile(designDocument->fileName());
+        currentProject = ProjectExplorer::ProjectTree::currentProject();
 
     if (!currentProject)
-        return 0;
-
-    if (!isFileInProject(designDocument, currentProject))
-        return 0;
-
-    designDocument->connect(projectExplorer, SIGNAL(currentProjectChanged(ProjectExplorer::Project*)), designDocument, SLOT(updateActiveQtVersion()), Qt::UniqueConnection);
-
-    designDocument->connect(currentProject, SIGNAL(activeTargetChanged(ProjectExplorer::Target*)), designDocument, SLOT(updateActiveQtVersion()), Qt::UniqueConnection);
+        return nullptr;
 
 
-    ProjectExplorer::Target *target = currentProject->activeTarget();
+    QObject::connect(ProjectTree::instance(), &ProjectTree::currentProjectChanged,
+                     designDocument, &DesignDocument::updateActiveQtVersion, Qt::UniqueConnection);
+
+    QObject::connect(currentProject, &Project::activeTargetChanged,
+                     designDocument, &DesignDocument::updateActiveQtVersion, Qt::UniqueConnection);
+
+    QObject::connect(ProjectTree::instance(), &ProjectTree::currentProjectChanged,
+                     designDocument, &DesignDocument::updateCurrentProject, Qt::UniqueConnection);
+
+    QObject::connect(currentProject, &Project::activeTargetChanged,
+                     designDocument, &DesignDocument::updateCurrentProject, Qt::UniqueConnection);
+
+
+    Target *target = currentProject->activeTarget();
 
     if (!target)
-        return 0;
+        return nullptr;
 
-    designDocument->connect(target, SIGNAL(kitChanged()), designDocument, SLOT(updateActiveQtVersion()), Qt::UniqueConnection);
+    if (!target->kit() || !target->kit()->isValid())
+        return nullptr;
+    QObject::connect(target, &Target::kitChanged,
+                     designDocument, &DesignDocument::updateActiveQtVersion, Qt::UniqueConnection);
 
     return target->kit();
 }
@@ -672,12 +655,18 @@ void DesignDocument::updateActiveQtVersion()
     viewManager().setNodeInstanceViewKit(m_currentKit);
 }
 
-QString DesignDocument::contextHelpId() const
+void DesignDocument::updateCurrentProject()
+{
+    ProjectExplorer::Project *currentProject = ProjectExplorer::SessionManager::projectForFile(fileName());
+    viewManager().setNodeInstanceViewProject(currentProject);
+}
+
+void DesignDocument::contextHelp(const Core::IContext::HelpCallback &callback) const
 {
     if (view())
-        view()->contextHelpId();
-
-    return QString();
+        view()->contextHelp(callback);
+    else
+        callback({});
 }
 
 } // namespace QmlDesigner
