@@ -47,9 +47,7 @@ using namespace Utils;
 namespace QmlJSEditor {
 namespace Internal {
 
-QmlTaskManager::QmlTaskManager(QObject *parent) :
-    QObject(parent),
-    m_updatingSemantic(false)
+QmlTaskManager::QmlTaskManager()
 {
     // displaying results incrementally leads to flickering
 //    connect(&m_messageCollector, &QFutureWatcherBase::resultsReadyAt,
@@ -90,7 +88,7 @@ void QmlTaskManager::collectMessages(
         QHash<QString, QList<DiagnosticMessage> > linkMessages;
         ContextPtr context;
         if (updateSemantic) {
-            QmlJS::Link link(snapshot, vContext, snapshot.libraryInfo(info.qtImportsPath));
+            QmlJS::Link link(snapshot, vContext, QmlJS::LibraryInfo());
             context = link(&linkMessages);
         }
 
