@@ -23,15 +23,17 @@
 **
 ****************************************************************************/
 
-#ifndef QMLPROFILERVIEWMANAGER_H
-#define QMLPROFILERVIEWMANAGER_H
+#pragma once
+
+#include "qmlprofilerstatisticsview.h"
+#include "qmlprofilertraceview.h"
+#include "flamegraphview.h"
 
 #include <QObject>
 
-namespace QmlProfiler {
-class QmlProfilerModelManager;
-class QmlProfilerStateManager;
+namespace Utils { class Perspective; }
 
+namespace QmlProfiler {
 namespace Internal {
 
 class QmlProfilerTool;
@@ -39,37 +41,36 @@ class QmlProfilerTool;
 class QmlProfilerViewManager : public QObject
 {
     Q_OBJECT
+
 public:
-    explicit QmlProfilerViewManager(QObject *parent,
-                                    QmlProfilerModelManager *modelManager,
-                                    QmlProfilerStateManager *profilerState);
-    ~QmlProfilerViewManager();
+    QmlProfilerViewManager(QObject *parent,
+                           QmlProfilerModelManager *modelManager,
+                           QmlProfilerStateManager *profilerState);
+    ~QmlProfilerViewManager() override;
 
-    void createViews();
+    QmlProfilerTraceView *traceView() const { return m_traceView; }
+    QmlProfilerStatisticsView *statisticsView() const { return m_statisticsView; }
+    FlameGraphView *flameGraphView() const { return m_flameGraphView; }
+    Utils::Perspective *perspective() const { return m_perspective; }
 
-    // used by the options "limit events to range"
-    bool hasValidSelection() const;
-    qint64 selectionStart() const;
-    qint64 selectionEnd() const;
-    bool isEventsRestrictedToRange() const;
-    void restrictEventsToRange(qint64 rangeStart, qint64 rangeEnd);
-
-    void raiseTimeline();
-
-public slots:
     void clear();
 
 signals:
     void typeSelected(int typeId);
     void gotoSourceLocation(QString,int,int);
+    void viewsCreated();
 
 private:
-    class QmlProfilerViewManagerPrivate;
-    QmlProfilerViewManagerPrivate *d;
+    void createViews();
+
+    QmlProfilerTraceView *m_traceView = nullptr;
+    QmlProfilerStatisticsView *m_statisticsView = nullptr;
+    FlameGraphView *m_flameGraphView = nullptr;
+    QmlProfilerStateManager *m_profilerState = nullptr;
+    QmlProfilerModelManager *m_profilerModelManager = nullptr;
+    Utils::Perspective *m_perspective = nullptr;
 };
 
 
 } // namespace Internal
 } // namespace QmlProfiler
-
-#endif // QMLPROFILERVIEWMANAGER_H
