@@ -23,19 +23,14 @@
 **
 ****************************************************************************/
 
-#ifndef REMOTELINUX_CONSTANTS_H
-#define REMOTELINUX_CONSTANTS_H
+#pragma once
 
 namespace RemoteLinux {
 namespace Constants {
 
 const char GenericLinuxOsType[] = "GenericLinuxOsType";
 
-const char GenericDeployKeyToDeviceActionId[] = "RemoteLinux.GenericDeployKeyToDeviceAction";
-
 const char EMBEDDED_LINUX_QT[] = "RemoteLinux.EmbeddedLinuxQt";
 
 } // Constants
 } // RemoteLinux
-
-#endif // REMOTELINUX_CONSTANTS_H
