@@ -39,7 +39,6 @@
 #include <vcsbase/basevcssubmiteditorfactory.h>
 #include <vcsbase/vcsbaseconstants.h>
 #include <vcsbase/vcsoutputwindow.h>
-#include <vcsbase/vcsbaseeditorparameterwidget.h>
 
 #include <texteditor/textdocument.h>
 
@@ -54,9 +53,9 @@
 #include <coreplugin/locator/commandlocator.h>
 #include <coreplugin/messagemanager.h>
 
+#include <utils/algorithm.h>
 #include <utils/fileutils.h>
 #include <utils/hostosinfo.h>
-#include <utils/mimetypes/mimedatabase.h>
 #include <utils/parameteraction.h>
 #include <utils/qtcassert.h>
 #include <utils/synchronousprocess.h>
@@ -75,7 +74,7 @@
 #include <QMessageBox>
 #include <QInputDialog>
 
-#include <limits.h>
+#include <climits>
 
 #ifdef WITH_TESTS
 #include <QTest>
@@ -124,8 +123,9 @@ const VcsBaseEditorParameters editorParameters[] = {
 // Utility to find a parameter set by type
 static const VcsBaseEditorParameters *findType(int ie)
 {
-    const EditorContentType et = static_cast<EditorContentType>(ie);
-    return VcsBaseEditor::findType(editorParameters, sizeof(editorParameters)/sizeof(editorParameters[0]), et);
+    return VcsBaseEditor::findType(editorParameters,
+                                   sizeof(editorParameters)/sizeof(*editorParameters),
+                                   static_cast<EditorContentType>(ie));
 }
 
 static inline QString debugCodec(const QTextCodec *c)
@@ -135,7 +135,7 @@ static inline QString debugCodec(const QTextCodec *c)
 
 // Parse "svn status" output for added/conflicted/deleted/modified files
 // "M<7blanks>file"
-typedef QList<SubversionSubmitEditor::StatusFilePair> StatusList;
+using StatusList = QList<SubversionSubmitEditor::StatusFilePair>;
 
 StatusList parseStatusOutput(const QString &output)
 {
@@ -169,7 +169,7 @@ static inline QStringList svnDirectories()
 }
 
 // ------------- SubversionPlugin
-SubversionPlugin *SubversionPlugin::m_subversionPluginInstance = 0;
+SubversionPlugin *SubversionPlugin::m_subversionPluginInstance = nullptr;
 
 SubversionPlugin::SubversionPlugin() :
     m_svnDirectories(svnDirectories())
@@ -211,28 +211,27 @@ bool SubversionPlugin::initialize(const QStringList & /*arguments */, QString *e
 
     Context context(SUBVERSION_CONTEXT);
 
-    initializeVcs(new SubversionControl(this), context);
+    initializeVcs<SubversionControl>(context, this);
 
     m_subversionPluginInstance = this;
 
-    Utils::MimeDatabase::addMimeTypes(QLatin1String(":/trolltech.subversion/Subversion.mimetypes.xml"));
-
     m_client = new SubversionClient;
 
-    addAutoReleasedObject(new SettingsPage(versionControl()));
+    new SettingsPage(versionControl(), this);
 
-    addAutoReleasedObject(new VcsSubmitEditorFactory(&submitParameters,
-        []() { return new SubversionSubmitEditor(&submitParameters); }));
+    new VcsSubmitEditorFactory(&submitParameters,
+        []() { return new SubversionSubmitEditor(&submitParameters); }, this);
 
-    static const char *describeSlot = SLOT(describe(QString,QString));
+    const auto describeFunc = [this](const QString &source, const QString &id) {
+        describe(source, id);
+    };
     const int editorCount = sizeof(editorParameters) / sizeof(editorParameters[0]);
     const auto widgetCreator = []() { return new SubversionEditorWidget; };
     for (int i = 0; i < editorCount; i++)
-        addAutoReleasedObject(new VcsEditorFactory(editorParameters + i, widgetCreator, this, describeSlot));
+        new VcsEditorFactory(editorParameters + i, widgetCreator, describeFunc, this);
 
     const QString prefix = QLatin1String("svn");
-    m_commandLocator = new CommandLocator("Subversion", prefix, prefix);
-    addAutoReleasedObject(m_commandLocator);
+    m_commandLocator = new CommandLocator("Subversion", prefix, prefix, this);
 
     // Register actions
     ActionContainer *toolsContainer = ActionManager::actionContainer(M_TOOLS);
@@ -247,7 +246,7 @@ bool SubversionPlugin::initialize(const QStringList & /*arguments */, QString *e
     command = ActionManager::registerAction(m_diffCurrentAction,
         CMD_ID_DIFF_CURRENT, context);
     command->setAttribute(Command::CA_UpdateText);
-    command->setDefaultKeySequence(QKeySequence(UseMacShortcuts ? tr("Meta+S,Meta+D") : tr("Alt+S,Alt+D")));
+    command->setDefaultKeySequence(QKeySequence(useMacShortcuts ? tr("Meta+S,Meta+D") : tr("Alt+S,Alt+D")));
     connect(m_diffCurrentAction, &QAction::triggered, this, &SubversionPlugin::diffCurrentFile);
     subversionMenu->addAction(command);
     m_commandLocator->appendCommand(command);
@@ -274,7 +273,7 @@ bool SubversionPlugin::initialize(const QStringList & /*arguments */, QString *e
     command = ActionManager::registerAction(m_addAction, CMD_ID_ADD,
         context);
     command->setAttribute(Command::CA_UpdateText);
-    command->setDefaultKeySequence(QKeySequence(UseMacShortcuts ? tr("Meta+S,Meta+A") : tr("Alt+S,Alt+A")));
+    command->setDefaultKeySequence(QKeySequence(useMacShortcuts ? tr("Meta+S,Meta+A") : tr("Alt+S,Alt+A")));
     connect(m_addAction, &QAction::triggered, this, &SubversionPlugin::addCurrentFile);
     subversionMenu->addAction(command);
     m_commandLocator->appendCommand(command);
@@ -283,7 +282,7 @@ bool SubversionPlugin::initialize(const QStringList & /*arguments */, QString *e
     command = ActionManager::registerAction(m_commitCurrentAction,
         CMD_ID_COMMIT_CURRENT, context);
     command->setAttribute(Command::CA_UpdateText);
-    command->setDefaultKeySequence(QKeySequence(UseMacShortcuts ? tr("Meta+S,Meta+C") : tr("Alt+S,Alt+C")));
+    command->setDefaultKeySequence(QKeySequence(useMacShortcuts ? tr("Meta+S,Meta+C") : tr("Alt+S,Alt+C")));
     connect(m_commitCurrentAction, &QAction::triggered, this, &SubversionPlugin::startCommitCurrentFile);
     subversionMenu->addAction(command);
     m_commandLocator->appendCommand(command);
@@ -388,26 +387,18 @@ bool SubversionPlugin::initialize(const QStringList & /*arguments */, QString *e
     subversionMenu->addAction(command);
     m_commandLocator->appendCommand(command);
 
-    // Actions of the submit editor
-    Context svncommitcontext(Constants::SUBVERSION_COMMIT_EDITOR_ID);
-
-    m_submitCurrentLogAction = new QAction(VcsBaseSubmitEditor::submitIcon(), tr("Commit"), this);
-    command = ActionManager::registerAction(m_submitCurrentLogAction, SUBMIT_CURRENT, svncommitcontext);
-    command->setAttribute(Command::CA_UpdateText);
-    connect(m_submitCurrentLogAction, &QAction::triggered, this, &SubversionPlugin::submitCurrentLog);
-
-    m_submitDiffAction = new QAction(VcsBaseSubmitEditor::diffIcon(), tr("Diff &Selected Files"), this);
-    ActionManager::registerAction(m_submitDiffAction , DIFF_SELECTED, svncommitcontext);
-
-    m_submitUndoAction = new QAction(tr("&Undo"), this);
-    ActionManager::registerAction(m_submitUndoAction, Core::Constants::UNDO, svncommitcontext);
-
-    m_submitRedoAction = new QAction(tr("&Redo"), this);
-    ActionManager::registerAction(m_submitRedoAction, Core::Constants::REDO, svncommitcontext);
-
     return true;
 }
 
+bool SubversionPlugin::isVcsDirectory(const FileName &fileName)
+{
+    const QString baseName = fileName.fileName();
+    return fileName.toFileInfo().isDir()
+            && contains(m_svnDirectories, [baseName](const QString &s) {
+        return !baseName.compare(s, HostOsInfo::fileNameCaseSensitivity());
+    });
+}
+
 SubversionClient *SubversionPlugin::client() const
 {
     QTC_CHECK(m_client);
@@ -419,7 +410,7 @@ bool SubversionPlugin::submitEditorAboutToClose()
     if (!isCommitEditorOpen())
         return true;
 
-    SubversionSubmitEditor *editor = qobject_cast<SubversionSubmitEditor *>(submitEditor());
+    auto editor = qobject_cast<SubversionSubmitEditor *>(submitEditor());
     QTC_ASSERT(editor, return true);
     IDocument *editorDocument = editor->document();
     QTC_ASSERT(editorDocument, return true);
@@ -434,12 +425,9 @@ bool SubversionPlugin::submitEditorAboutToClose()
     // Prompt user. Force a prompt unless submit was actually invoked (that
     // is, the editor was closed or shutdown).
     VcsBaseClientSettings &newSettings = client()->settings();
-    const VcsBaseSubmitEditor::PromptSubmitResult answer =
-            editor->promptSubmit(tr("Closing Subversion Editor"),
-                                 tr("Do you want to commit the change?"),
-                                 tr("The commit message check failed. Do you want to commit the change?"),
-                                 newSettings.boolPointer(SubversionSettings::promptOnSubmitKey),
-                                 !m_submitActionTriggered);
+    const VcsBaseSubmitEditor::PromptSubmitResult answer = editor->promptSubmit(
+                this, newSettings.boolPointer(SubversionSettings::promptOnSubmitKey),
+                !m_submitActionTriggered);
     m_submitActionTriggered = false;
     switch (answer) {
     case VcsBaseSubmitEditor::SubmitCanceled:
@@ -454,15 +442,10 @@ bool SubversionPlugin::submitEditorAboutToClose()
     bool closeEditor = true;
     if (!fileList.empty()) {
         // get message & commit
-        closeEditor = DocumentManager::saveDocument(editorDocument);
-        if (closeEditor) {
-            VcsCommand *commitCmd = m_client->createCommitCmd(m_commitRepository,
-                                                              fileList,
-                                                              m_commitMessageFileName);
-            QObject::connect(commitCmd, &VcsCommand::finished,
-                             this, [this]() { cleanCommitMessageFile(); });
-            commitCmd->execute();
-        }
+        closeEditor = DocumentManager::saveDocument(editorDocument)
+                && m_client->doCommit(m_commitRepository, fileList, m_commitMessageFileName);
+        if (closeEditor)
+            cleanCommitMessageFile();
     }
     return closeEditor;
 }
@@ -475,10 +458,9 @@ void SubversionPlugin::diffCommitFiles(const QStringList &files)
 SubversionSubmitEditor *SubversionPlugin::openSubversionSubmitEditor(const QString &fileName)
 {
     IEditor *editor = EditorManager::openEditor(fileName, Constants::SUBVERSION_COMMIT_EDITOR_ID);
-    SubversionSubmitEditor *submitEditor = qobject_cast<SubversionSubmitEditor*>(editor);
-    QTC_ASSERT(submitEditor, return 0);
+    auto submitEditor = qobject_cast<SubversionSubmitEditor*>(editor);
+    QTC_ASSERT(submitEditor, return nullptr);
     setSubmitEditor(submitEditor);
-    submitEditor->registerActions(m_submitUndoAction, m_submitRedoAction, m_submitCurrentLogAction, m_submitDiffAction);
     connect(submitEditor, &VcsBaseSubmitEditor::diffSelectedFiles,
             this, &SubversionPlugin::diffCommitFiles);
     submitEditor->setCheckScriptWorkingDirectory(m_commitRepository);
@@ -559,7 +541,7 @@ void SubversionPlugin::revertCurrentFile()
 
     QStringList args(QLatin1String("diff"));
     args << SubversionClient::addAuthenticationOptions(client()->settings());
-    args.push_back(state.relativeCurrentFile());
+    args.push_back(SubversionClient::escapeFile(state.relativeCurrentFile()));
 
     const SubversionResponse diffResponse
             = runSvn(state.currentFileTopLevel(), args, m_client->vcsTimeoutS(), 0);
@@ -580,7 +562,7 @@ void SubversionPlugin::revertCurrentFile()
     args.clear();
     args << QLatin1String("revert");
     args << SubversionClient::addAuthenticationOptions(client()->settings());
-    args << state.relativeCurrentFile();
+    args << SubversionClient::escapeFile(state.relativeCurrentFile());
 
     const SubversionResponse revertResponse
             = runSvn(state.currentFileTopLevel(), args, m_client->vcsTimeoutS(),
@@ -634,6 +616,9 @@ void SubversionPlugin::startCommitProject()
  * commit will start. */
 void SubversionPlugin::startCommit(const QString &workingDir, const QStringList &files)
 {
+    if (!promptBeforeCommit())
+        return;
+
     if (raiseSubmitEditor())
         return;
     if (isCommitEditorOpen()) {
@@ -643,7 +628,7 @@ void SubversionPlugin::startCommit(const QString &workingDir, const QStringList
 
     QStringList args(QLatin1String("status"));
     args << SubversionClient::addAuthenticationOptions(client()->settings());
-    args += files;
+    args += SubversionClient::escapeFiles(files);
 
     const SubversionResponse response
             = runSvn(workingDir, args, m_client->vcsTimeoutS(), 0);
@@ -724,7 +709,7 @@ void SubversionPlugin::svnStatus(const QString &workingDir, const QString &relat
     QStringList args(QLatin1String("status"));
     args << SubversionClient::addAuthenticationOptions(client()->settings());
     if (!relativePath.isEmpty())
-        args.append(relativePath);
+        args.append(SubversionClient::escapeFile(relativePath));
     VcsOutputWindow::setRepository(workingDir);
     runSvn(workingDir, args, m_client->vcsTimeoutS(),
            VcsCommand::ShowStdOut | VcsCommand::ShowSuccessMessage);
@@ -766,14 +751,6 @@ void SubversionPlugin::annotateCurrentFile()
     vcsAnnotate(state.currentFileTopLevel(), state.relativeCurrentFile());
 }
 
-void SubversionPlugin::annotateVersion(const QString &workingDirectory,
-                                       const QString &file,
-                                       const QString &revision,
-                                       int lineNr)
-{
-    vcsAnnotate(workingDirectory, file, revision, lineNr);
-}
-
 void SubversionPlugin::vcsAnnotate(const QString &workingDir, const QString &file,
                                    const QString &revision /* = QString() */,
                                    int lineNumber /* = -1 */)
@@ -789,7 +766,7 @@ void SubversionPlugin::vcsAnnotate(const QString &workingDir, const QString &fil
     if (!revision.isEmpty())
         args << QLatin1String("-r") << revision;
     args.push_back(QLatin1String("-v"));
-    args.append(QDir::toNativeSeparators(file));
+    args.append(QDir::toNativeSeparators(SubversionClient::escapeFile(file)));
 
     const SubversionResponse response
             = runSvn(workingDir, args, m_client->vcsTimeoutS(),
@@ -865,7 +842,7 @@ void SubversionPlugin::slotDescribe()
     describe(state.topLevel(), QString::number(revision));
 }
 
-void SubversionPlugin::submitCurrentLog()
+void SubversionPlugin::commitFromEditor()
 {
     m_submitActionTriggered = true;
     QTC_ASSERT(submitEditor(), return);
@@ -900,18 +877,17 @@ IEditor *SubversionPlugin::showOutputInEditor(const QString &title, const QStrin
                                                      QTextCodec *codec)
 {
     const VcsBaseEditorParameters *params = findType(editorType);
-    QTC_ASSERT(params, return 0);
+    QTC_ASSERT(params, return nullptr);
     const Id id = params->id;
     if (Subversion::Constants::debug)
         qDebug() << "SubversionPlugin::showOutputInEditor" << title << id.name()
                  <<  "Size= " << output.size() <<  " Type=" << editorType << debugCodec(codec);
     QString s = title;
     IEditor *editor = EditorManager::openEditorWithContents(id, &s, output.toUtf8());
-    connect(editor, SIGNAL(annotateRevisionRequested(QString,QString,QString,int)),
-            this, SLOT(annotateVersion(QString,QString,QString,int)));
-    SubversionEditorWidget *e = qobject_cast<SubversionEditorWidget*>(editor->widget());
+    auto e = qobject_cast<SubversionEditorWidget*>(editor->widget());
     if (!e)
-        return 0;
+        return nullptr;
+    connect(e, &VcsBaseEditorWidget::annotateRevisionRequested, this, &SubversionPlugin::vcsAnnotate);
     e->setForceReadOnly(true);
     s.replace(QLatin1Char(' '), QLatin1Char('_'));
     e->textDocument()->setFallbackSaveAsFileName(s);
@@ -949,7 +925,7 @@ QString SubversionPlugin::synchronousTopic(const QString &repository) const
 
 bool SubversionPlugin::vcsAdd(const QString &workingDir, const QString &rawFileName)
 {
-    const QString file = QDir::toNativeSeparators(rawFileName);
+    const QString file = QDir::toNativeSeparators(SubversionClient::escapeFile(rawFileName));
     QStringList args;
     args << QLatin1String("add")
          << SubversionClient::addAuthenticationOptions(client()->settings())
@@ -962,7 +938,7 @@ bool SubversionPlugin::vcsAdd(const QString &workingDir, const QString &rawFileN
 
 bool SubversionPlugin::vcsDelete(const QString &workingDir, const QString &rawFileName)
 {
-    const QString file = QDir::toNativeSeparators(rawFileName);
+    const QString file = QDir::toNativeSeparators(SubversionClient::escapeFile(rawFileName));
 
     QStringList args;
     args << QLatin1String("delete");
@@ -979,7 +955,8 @@ bool SubversionPlugin::vcsMove(const QString &workingDir, const QString &from, c
 {
     QStringList args(QLatin1String("move"));
     args << SubversionClient::addAuthenticationOptions(client()->settings());
-    args << QDir::toNativeSeparators(from) << QDir::toNativeSeparators(to);
+    args << QDir::toNativeSeparators(SubversionClient::escapeFile(from))
+         << QDir::toNativeSeparators(SubversionClient::escapeFile(to));
     const SubversionResponse response
             = runSvn(workingDir, args, m_client->vcsTimeoutS(),
                      VcsCommand::SshPasswordPrompt | VcsCommand::ShowStdOut
@@ -1042,7 +1019,8 @@ bool SubversionPlugin::managesFile(const QString &workingDirectory, const QStrin
 {
     QStringList args;
     args << QLatin1String("status");
-    args << SubversionClient::addAuthenticationOptions(client()->settings()) << fileName;
+    args << SubversionClient::addAuthenticationOptions(client()->settings())
+         << QDir::toNativeSeparators(SubversionClient::escapeFile(fileName));
     SubversionResponse response
             = runSvn(workingDirectory, args, m_client->vcsTimeoutS(), 0);
     return response.stdOut.isEmpty() || response.stdOut.at(0) != QLatin1Char('?');
