@@ -92,7 +92,7 @@ def main():
         return
     for _ in range(5):
         type(editorArea, "<Left>")
-    invokeContextMenuItem(editorArea, "Find Usages")
+    invokeContextMenuItem(editorArea, "Find References to Symbol Under Cursor")
     # check if usage was properly found
     expectedResults = [ExpectedResult("color-animation.qml", 49, "Rectangle {"),
                        ExpectedResult("color-animation.qml", 109, "Rectangle {"),
@@ -111,7 +111,7 @@ def main():
         return
     for _ in range(87):
         type(editorArea, "<Left>")
-    invokeMenuItem("Tools", "QML/JS", "Find Usages")
+    invokeMenuItem("Tools", "QML/JS", "Find References to Symbol Under Cursor")
     # check if usage was properly found
     expectedResults = [ExpectedResult("color-animation.qml", 50, "anchors { left: parent.left; top: parent.top; right: parent.right; bottom: parent.verticalCenter }"),
                        ExpectedResult("color-animation.qml", 110, "anchors { left: parent.left; top: parent.verticalCenter; right: parent.right; bottom: parent.bottom }"),
